"""
Test AArch64 dynamic register sets
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class RegisterCommandsTestCase(TestBase):
    def check_sve_register_size(self, set, name, expected):
        reg_value = set.GetChildMemberWithName(name)
        self.assertTrue(reg_value.IsValid(), "Expected a register named %s" % (name))
        self.assertEqual(
            reg_value.GetByteSize(),
            expected,
            "Expected a register %s size == %i bytes" % (name, expected),
        )

    def sve_regs_read_dynamic(self, sve_registers):
        vg_reg = sve_registers.GetChildMemberWithName("vg")
        vg_reg_value = sve_registers.GetChildMemberWithName("vg").GetValueAsUnsigned()

        z_reg_size = vg_reg_value * 8
        p_reg_size = int(z_reg_size / 8)

        for i in range(32):
            z_regs_value = (
                "{"
                + " ".join("0x{:02x}".format(i + 1) for _ in range(z_reg_size))
                + "}"
            )
            self.expect("register read z%i" % (i), substrs=[z_regs_value])

        # Set P registers with random test values. The P registers are predicate
        # registers, which hold one bit for each byte available in a Z register.
        # For below mentioned values of P registers, P(0,5,10,15) will have all
        # Z register lanes set while P(4,9,14) will have no lanes set.
        p_value_bytes = ["0xff", "0x55", "0x11", "0x01", "0x00"]
        for i in range(16):
            p_regs_value = (
                "{" + " ".join(p_value_bytes[i % 5] for _ in range(p_reg_size)) + "}"
            )
            self.expect("register read p%i" % (i), substrs=[p_regs_value])

        self.expect("register read ffr", substrs=[p_regs_value])

        for i in range(32):
            z_regs_value = (
                "{"
                + " ".join("0x{:02x}".format(32 - i) for _ in range(z_reg_size))
                + "}"
            )
            self.runCmd("register write z%i '%s'" % (i, z_regs_value))
            self.expect("register read z%i" % (i), substrs=[z_regs_value])

        for i in range(16):
            p_regs_value = (
                "{"
                + " ".join("0x{:02x}".format(16 - i) for _ in range(p_reg_size))
                + "}"
            )
            self.runCmd("register write p%i '%s'" % (i, p_regs_value))
            self.expect("register read p%i" % (i), substrs=[p_regs_value])

        p_regs_value = (
            "{" + " ".join("0x{:02x}".format(8) for _ in range(p_reg_size)) + "}"
        )
        self.runCmd("register write ffr " + "'" + p_regs_value + "'")
        self.expect("register read ffr", substrs=[p_regs_value])

    def setup_register_config_test(self, run_args=None):
        self.build()
        self.line = line_number("main.c", "// Set a break point here.")

        exe = self.getBuildArtifact("a.out")
        if run_args is not None:
            self.runCmd("settings set target.run-args " + run_args)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        return self.thread().GetSelectedFrame().GetRegisters()

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_aarch64_dynamic_regset_config(self):
        """Test AArch64 Dynamic Register sets configuration."""
        if not self.isAArch64SVE():
            self.skipTest("SVE must be present")

        register_sets = self.setup_register_config_test()

        for registerSet in register_sets:
            if "Scalable Vector Extension Registers" in registerSet.GetName():
                self.assertTrue(
                    self.isAArch64SVE(),
                    "LLDB enabled AArch64 SVE register set when it was disabled by target.",
                )
                self.sve_regs_read_dynamic(registerSet)
            if "MTE Control Register" in registerSet.GetName():
                self.assertTrue(
                    self.isAArch64MTE(),
                    "LLDB enabled AArch64 MTE register set when it was disabled by target.",
                )
                self.runCmd("register write mte_ctrl 0x7fff9")
                self.expect(
                    "register read mte_ctrl", substrs=["mte_ctrl = 0x000000000007fff9"]
                )
            if "Pointer Authentication Registers" in registerSet.GetName():
                self.assertTrue(
                    self.isAArch64PAuth(),
                    "LLDB enabled AArch64 Pointer Authentication register set when it was disabled by target.",
                )
                self.expect("register read data_mask", substrs=["data_mask = 0x"])
                self.expect("register read code_mask", substrs=["code_mask = 0x"])
            if "Scalable Matrix Extension Registers" in registerSet.GetName():
                self.assertTrue(
                    self.isAArch64SME(),
                    "LLDB Enabled SME register set when it was disabled by target",
                )

    def make_za_value(self, vl, generator):
        # Generate a vector value string "{0x00 0x01....}".
        rows = []
        for row in range(vl):
            byte = "0x{:02x}".format(generator(row))
            rows.append(" ".join([byte] * vl))
        return "{" + " ".join(rows) + "}"

    def make_zt0_value(self, generator):
        num_bytes = 512 // 8
        elements = []
        for i in range(num_bytes):
            elements.append("0x{:02x}".format(generator(i)))

        return "{" + " ".join(elements) + "}"

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_aarch64_dynamic_regset_config_sme(self):
        """Test AArch64 Dynamic Register sets configuration, but only SME
        registers."""
        if not self.isAArch64SMEFA64():
            self.skipTest("SME and the smefa64 extension must be present")

        register_sets = self.setup_register_config_test("sme")

        ssve_registers = register_sets.GetFirstValueByName(
            "Scalable Vector Extension Registers"
        )
        self.assertTrue(ssve_registers.IsValid())
        self.sve_regs_read_dynamic(ssve_registers)

        sme_registers = register_sets.GetFirstValueByName(
            "Scalable Matrix Extension Registers"
        )
        self.assertTrue(sme_registers.IsValid())

        vg = ssve_registers.GetChildMemberWithName("vg").GetValueAsUnsigned()
        vl = vg * 8
        # When first enabled it is all 0s.
        self.expect("register read za", substrs=[self.make_za_value(vl, lambda r: 0)])
        za_value = self.make_za_value(vl, lambda r: r + 1)
        self.runCmd("register write za '{}'".format(za_value))
        self.expect("register read za", substrs=[za_value])

        # SVG should match VG because we're in streaming mode.

        self.assertTrue(sme_registers.IsValid())
        svg = sme_registers.GetChildMemberWithName("svg").GetValueAsUnsigned()
        self.assertEqual(vg, svg)

        # SVCR should be SVCR.SM | SVCR.ZA aka 3 because streaming mode is on
        # and ZA is enabled.
        svcr = sme_registers.GetChildMemberWithName("svcr").GetValueAsUnsigned()
        self.assertEqual(3, svcr)

        # SVCR is read only so we do not test writing to it.

    def write_to_enable_za_test(self, has_zt0, write_za_first):
        # Run a test where we start with ZA disabled, and write to either ZA
        # or ZT0 which causes them to become enabled.

        # No argument, so ZA and ZT0 will be disabled when we break.
        register_sets = self.setup_register_config_test()

        # vg is the non-streaming vg as we are in non-streaming mode, so we need
        # to use svg.
        sme_registers = register_sets.GetFirstValueByName(
            "Scalable Matrix Extension Registers"
        )
        self.assertTrue(sme_registers.IsValid())
        svg = sme_registers.GetChildMemberWithName("svg").GetValueAsUnsigned()

        # We are not in streaming mode, ZA is disabled, so this should be 0.
        svcr = sme_registers.GetChildMemberWithName("svcr").GetValueAsUnsigned()
        self.assertEqual(0, svcr)

        svl = svg * 8
        # A disabled ZA is shown as all 0s.
        disabled_za = self.make_za_value(svl, lambda r: 0)
        self.expect("register read za", substrs=[disabled_za])

        disabled_zt0 = self.make_zt0_value(lambda n: 0)
        if has_zt0:
            # A disabled zt0 is all 0s.
            self.expect("register read zt0", substrs=[disabled_zt0])

        # Writing to ZA or ZTO enables both and we should be able to read the
        # value back.
        za_value = self.make_za_value(svl, lambda r: r + 1)
        zt0_value = self.make_zt0_value(lambda n: n + 1)

        if write_za_first:
            # This enables ZA and ZT0.
            self.runCmd("register write za '{}'".format(za_value))
            self.expect("register read za", substrs=[za_value])

            if has_zt0:
                # ZT0 is still 0s at this point, though it is active.
                self.expect("register read zt0", substrs=[disabled_zt0])

                # Now write ZT0 to we can check it reads back correctly.
                self.runCmd("register write zt0 '{}'".format(zt0_value))
                self.expect("register read zt0", substrs=[zt0_value])
        else:
            if not has_zt0:
                self.fail("Cannot write to zt0 when sme2 is not present.")

            # Instead use the write of ZT0 to activate ZA.
            self.runCmd("register write zt0 '{}'".format(zt0_value))
            self.expect("register read zt0", substrs=[zt0_value])

            # ZA will be active but 0s at this point, but it is active.
            self.expect("register read zt0", substrs=[disabled_za])

            # Write and read back ZA.
            self.runCmd("register write za '{}'".format(za_value))
            self.expect("register read za", substrs=[za_value])

        # Now SVCR.ZA should be set, which is bit 1.
        self.expect("register read svcr", substrs=["0x0000000000000002"])

        # SVCR is read only so we do not test writing to it.

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_aarch64_dynamic_regset_config_sme_write_za_to_enable(self):
        """Test that ZA and ZT0 (if present) shows as 0s when disabled and
        can be enabled by writing to ZA."""
        if not self.isAArch64SVE():
            self.skipTest("SVE must be present.")
        if not self.isAArch64SME():
            self.skipTest("SME must be present.")

        self.write_to_enable_za_test(self.isAArch64SME2(), True)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_aarch64_dynamic_regset_config_sme_write_zt0_to_enable(self):
        """Test that ZA and ZT0 (if present) shows as 0s when disabled and
        can be enabled by writing to ZT0."""
        if not self.isAArch64SVE():
            self.skipTest("SVE must be present.")
        if not self.isAArch64SME():
            self.skipTest("SME must be present.")
        if not self.isAArch64SME2():
            self.skipTest("SME2 must be present.")

        self.write_to_enable_za_test(True, True)



"""
Test the 'register' command.
"""

import os
import sys
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class RegisterCommandsTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.has_teardown = False

    def tearDown(self):
        self.dbg.GetSelectedTarget().GetProcess().Destroy()
        TestBase.tearDown(self)

    @skipIfiOSSimulator
    @skipIf(archs=no_match(["amd64", "arm$", "i386", "x86_64"]))
    @expectedFailureAll(oslist=["freebsd", "netbsd"], bugnumber="llvm.org/pr48371")
    def test_register_commands(self):
        """Test commands related to registers, in particular vector registers."""
        self.build()
        self.common_setup()

        # verify that logging does not assert
        self.log_enable("registers")

        error_str_matched = False
        if self.isAArch64SME() and self.platformIsDarwin():
            # On Darwin AArch64 SME machines, we will have unavailable
            # registers when not in Streaming SVE Mode/SME, so
            # `register read -a` will report that some registers
            # could not be read.  This is expected.
            error_str_matched = True

        if self.getArchitecture() == "x86_64" and self.platformIsDarwin():
            # debugserver on x86 will provide ds/es/ss/gsbase when the
            # kernel provides them, but most of the time they will be
            # unavailable.  So "register read -a" will report that
            # 4 registers were unavailable, it is expected.
            error_str_matched = True

        self.expect(
            "register read -a",
            MISSING_EXPECTED_REGISTERS,
            substrs=["registers were unavailable"],
            matching=error_str_matched,
        )

        all_registers = self.res.GetOutput()

        if self.getArchitecture() in ["amd64", "i386", "x86_64"]:
            self.runCmd("register read xmm0")
            if "ymm15 = " in all_registers:
                self.runCmd("register read ymm15")  # may be available
            if "bnd0 = " in all_registers:
                self.runCmd("register read bnd0")  # may be available
        elif self.getArchitecture() in [
            "arm",
            "armv7",
            "armv7k",
            "arm64",
            "arm64e",
            "arm64_32",
        ]:
            self.runCmd("register read s0")
            if "q15 = " in all_registers:
                self.runCmd("register read q15")  # may be available

        self.expect(
            "register read -s 8", substrs=["invalid register set index: 8"], error=True
        )

    @skipIfiOSSimulator
    # Writing of mxcsr register fails, presumably due to a kernel/hardware
    # problem
    @skipIfTargetAndroid(archs=["i386"])
    @skipIf(archs=no_match(["amd64", "arm$", "i386", "x86_64"]))
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr37995")
    def test_fp_register_write(self):
        """Test commands that write to registers, in particular floating-point registers."""
        self.build()
        self.fp_register_write()

    @skipIfiOSSimulator
    # "register read fstat" always return 0xffff
    @expectedFailureAndroid(archs=["i386"])
    @skipIf(archs=no_match(["amd64", "i386", "x86_64"]))
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr37995")
    def test_fp_special_purpose_register_read(self):
        """Test commands that read fpu special purpose registers."""
        self.build()
        self.fp_special_purpose_register_read()

    @skipIfiOSSimulator
    @skipIf(archs=no_match(["amd64", "arm$", "i386", "x86_64"]))
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr37683")
    def test_register_expressions(self):
        """Test expression evaluation with commands related to registers."""
        self.build()
        self.common_setup()

        if self.getArchitecture() in ["amd64", "i386", "x86_64"]:
            gpr = "eax"
            vector = "xmm0"
        elif self.getArchitecture() in ["arm64", "aarch64", "arm64e", "arm64_32"]:
            gpr = "w0"
            vector = "v0"
        elif self.getArchitecture() in ["arm", "armv7", "armv7k"]:
            gpr = "r0"
            vector = "q0"

        self.expect("expr/x $%s" % gpr, substrs=["unsigned int", " = 0x"])
        self.expect("expr $%s" % vector, substrs=["vector_type"])
        self.expect("expr (unsigned int)$%s[0]" % vector, substrs=["unsigned int"])

        if self.getArchitecture() in ["amd64", "x86_64"]:
            self.expect("expr -- ($rax & 0xffffffff) == $eax", substrs=["true"])

    @skipIfiOSSimulator
    @skipIf(archs=no_match(["amd64", "x86_64"]))
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr37683")
    def test_convenience_registers(self):
        """Test convenience registers."""
        self.build()
        self.convenience_registers()

    @skipIfiOSSimulator
    @skipIf(archs=no_match(["amd64", "x86_64"]))
    def test_convenience_registers_with_process_attach(self):
        """Test convenience registers after a 'process attach'."""
        self.build()
        self.convenience_registers_with_process_attach(test_16bit_regs=False)

    @skipIfiOSSimulator
    @skipIf(archs=no_match(["amd64", "x86_64"]))
    def test_convenience_registers_16bit_with_process_attach(self):
        """Test convenience registers after a 'process attach'."""
        self.build()
        self.convenience_registers_with_process_attach(test_16bit_regs=True)

    def common_setup(self):
        exe = self.getBuildArtifact("a.out")

        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break in main().
        lldbutil.run_break_set_by_symbol(self, "main", num_expected_locations=-1)

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

    # platform specific logging of the specified category
    def log_enable(self, category):
        # This intentionally checks the host platform rather than the target
        # platform as logging is host side.
        self.platform = ""
        if (
            sys.platform.startswith("freebsd")
            or sys.platform.startswith("linux")
            or sys.platform.startswith("netbsd")
        ):
            self.platform = "posix"

        if self.platform != "":
            self.log_file = self.getBuildArtifact("TestRegisters.log")
            self.runCmd(
                "log enable "
                + self.platform
                + " "
                + str(category)
                + " registers -v -f "
                + self.log_file,
                RUN_SUCCEEDED,
            )
            if not self.has_teardown:

                def remove_log(self):
                    if os.path.exists(self.log_file):
                        os.remove(self.log_file)

                self.has_teardown = True
                self.addTearDownHook(remove_log)

    def write_and_read(self, frame, register, new_value, must_exist=True):
        value = frame.FindValue(register, lldb.eValueTypeRegister)
        if must_exist:
            self.assertTrue(value.IsValid(), "finding a value for register " + register)
        elif not value.IsValid():
            return  # If register doesn't exist, skip this test

        # Also test the 're' alias.
        self.runCmd("re write " + register + " '" + new_value + "'")
        self.expect("register read " + register, substrs=[register + " = ", new_value])

    # This test relies on ftag containing the 'abridged' value.  Linux
    # and *BSD targets have been ported to report the full value instead
    # consistently with GDB.  They are covered by the new-style
    # lldb/test/Shell/Register/x86*-fp-read.test.
    @skipUnlessDarwin
    def fp_special_purpose_register_read(self):
        target = self.createTestTarget()

        # Launch the process and stop.
        self.expect("run", PROCESS_STOPPED, substrs=["stopped"])

        # Check stop reason; Should be either signal SIGTRAP or EXC_BREAKPOINT
        output = self.res.GetOutput()
        matched = False
        substrs = ["stop reason = EXC_BREAKPOINT", "stop reason = signal SIGTRAP"]
        for str1 in substrs:
            matched = output.find(str1) != -1
            with recording(self, False) as sbuf:
                print("%s sub string: %s" % ("Expecting", str1), file=sbuf)
                print("Matched" if matched else "Not Matched", file=sbuf)
            if matched:
                break
        self.assertTrue(matched, STOPPED_DUE_TO_SIGNAL)

        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        thread = process.GetThreadAtIndex(0)
        self.assertTrue(thread.IsValid(), "current thread is valid")

        currentFrame = thread.GetFrameAtIndex(0)
        self.assertTrue(currentFrame.IsValid(), "current frame is valid")

        # Extract the value of fstat and ftag flag at the point just before
        # we start pushing floating point values on st% register stack
        value = currentFrame.FindValue("fstat", lldb.eValueTypeRegister)
        error = lldb.SBError()
        reg_value_fstat_initial = value.GetValueAsUnsigned(error, 0)

        self.assertSuccess(error, "reading a value for fstat")
        value = currentFrame.FindValue("ftag", lldb.eValueTypeRegister)
        error = lldb.SBError()
        reg_value_ftag_initial = value.GetValueAsUnsigned(error, 0)

        self.assertSuccess(error, "reading a value for ftag")
        fstat_top_pointer_initial = (reg_value_fstat_initial & 0x3800) >> 11

        # Execute 'si' aka 'thread step-inst' instruction 5 times and with
        # every execution verify the value of fstat and ftag registers
        for x in range(0, 5):
            # step into the next instruction to push a value on 'st' register
            # stack
            self.runCmd("si", RUN_SUCCEEDED)

            # Verify fstat and save it to be used for verification in next
            # execution of 'si' command
            if not (reg_value_fstat_initial & 0x3800):
                self.expect(
                    "register read fstat",
                    substrs=[
                        "fstat" + " = ",
                        str(
                            "0x%0.4x" % ((reg_value_fstat_initial & ~(0x3800)) | 0x3800)
                        ),
                    ],
                )
                reg_value_fstat_initial = (reg_value_fstat_initial & ~(0x3800)) | 0x3800
                fstat_top_pointer_initial = 7
            else:
                self.expect(
                    "register read fstat",
                    substrs=[
                        "fstat" + " = ",
                        str("0x%0.4x" % (reg_value_fstat_initial - 0x0800)),
                    ],
                )
                reg_value_fstat_initial = reg_value_fstat_initial - 0x0800
                fstat_top_pointer_initial -= 1

            # Verify ftag and save it to be used for verification in next
            # execution of 'si' command
            self.expect(
                "register read ftag",
                substrs=[
                    "ftag" + " = ",
                    str(
                        "0x%0.4x"
                        % (reg_value_ftag_initial | (1 << fstat_top_pointer_initial))
                    ),
                ],
            )
            reg_value_ftag_initial = reg_value_ftag_initial | (
                1 << fstat_top_pointer_initial
            )

    def fp_register_write(self):
        target = self.createTestTarget()

        # Launch the process, stop at the entry point.
        error = lldb.SBError()
        flags = target.GetLaunchInfo().GetLaunchFlags()
        process = target.Launch(
            lldb.SBListener(),
            None,
            None,  # argv, envp
            None,
            None,
            None,  # stdin/out/err
            self.get_process_working_directory(),
            flags,  # launch flags
            True,  # stop at entry
            error,
        )
        self.assertSuccess(error, "Launch succeeds")

        self.assertEqual(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        thread = process.GetThreadAtIndex(0)
        self.assertTrue(thread.IsValid(), "current thread is valid")

        currentFrame = thread.GetFrameAtIndex(0)
        self.assertTrue(currentFrame.IsValid(), "current frame is valid")

        if self.getArchitecture() in ["amd64", "i386", "x86_64"]:
            reg_list = [
                # reg          value        must-have
                ("fcw", "0x0000ff0e", False),
                ("fsw", "0x0000ff0e", False),
                ("ftw", "0x0000ff0e", False),
                ("ip", "0x0000ff0e", False),
                ("dp", "0x0000ff0e", False),
                ("mxcsr", "0x0000ff0e", False),
                ("mxcsrmask", "0x0000ff0e", False),
            ]

            st0regname = None
            # Darwin is using stmmN by default but support stN as an alias.
            # Therefore, we need to check for stmmN first.
            if currentFrame.FindRegister("stmm0").IsValid():
                st0regname = "stmm0"
            elif currentFrame.FindRegister("st0").IsValid():
                st0regname = "st0"
            if st0regname is not None:
                # reg          value
                # must-have
                reg_list.append(
                    (
                        st0regname,
                        "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x00 0x00}",
                        True,
                    )
                )
                reg_list.append(
                    (
                        "xmm0",
                        "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x2f 0x2f}",
                        True,
                    )
                )
                reg_list.append(
                    (
                        "xmm15",
                        "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x0e 0x0f}",
                        False,
                    )
                )
        elif self.getArchitecture() in ["arm64", "aarch64", "arm64e", "arm64_32"]:
            reg_list = [
                # reg      value
                # must-have
                ("fpsr", "0xfbf79f9f", True),
                ("s0", "1.25", True),
                ("s31", "0.75", True),
                ("d1", "123", True),
                ("d17", "987", False),
                (
                    "v1",
                    "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x2f 0x2f}",
                    True,
                ),
                (
                    "v14",
                    "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x0e 0x0f}",
                    False,
                ),
            ]
        elif self.getArchitecture() in ["armv7"] and self.platformIsDarwin():
            reg_list = [
                # reg      value
                # must-have
                ("fpsr", "0xfbf79f9f", True),
                ("s0", "1.25", True),
                ("s31", "0.75", True),
                ("d1", "123", True),
                ("d17", "987", False),
                (
                    "q1",
                    "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x2f 0x2f}",
                    True,
                ),
                (
                    "q14",
                    "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x0e 0x0f}",
                    False,
                ),
            ]
        elif self.getArchitecture() in ["arm", "armv7k"]:
            reg_list = [
                # reg      value
                # must-have
                ("fpscr", "0xfbf79f9f", True),
                ("s0", "1.25", True),
                ("s31", "0.75", True),
                ("d1", "123", True),
                ("d17", "987", False),
                (
                    "q1",
                    "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x2f 0x2f}",
                    True,
                ),
                (
                    "q14",
                    "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x0e 0x0f}",
                    False,
                ),
            ]

        for reg, val, must in reg_list:
            self.write_and_read(currentFrame, reg, val, must)

        if self.getArchitecture() in ["amd64", "i386", "x86_64"]:
            if st0regname is None:
                self.fail("st0regname could not be determined")
            self.runCmd(
                "register write "
                + st0regname
                + ' "{0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00}"'
            )
            self.expect(
                "register read " + st0regname + " --format f",
                substrs=[st0regname + " = 0"],
            )

            # Check if AVX/MPX registers are defined at all.
            registerSets = currentFrame.GetRegisters()
            registers = frozenset(
                reg.GetName() for registerSet in registerSets for reg in registerSet
            )
            has_avx_regs = "ymm0" in registers
            has_mpx_regs = "bnd0" in registers
            # Check if they are actually present.
            self.runCmd("register read -a")
            output = self.res.GetOutput()
            has_avx = "ymm0 =" in output
            has_mpx = "bnd0 =" in output

            if has_avx:
                new_value = "{0x01 0x02 0x03 0x00 0x00 0x00 0x00 0x00 0x09 0x0a 0x2f 0x2f 0x2f 0x2f 0x0e 0x0f 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x0c 0x0d 0x0e 0x0f}"
                self.write_and_read(currentFrame, "ymm0", new_value)
                self.write_and_read(currentFrame, "ymm7", new_value)
                self.expect("expr $ymm0", substrs=["vector_type"])
            elif has_avx_regs:
                self.expect("register read ymm0", substrs=["error: unavailable"])
            else:
                self.expect(
                    "register read ymm0",
                    substrs=["Invalid register name 'ymm0'"],
                    error=True,
                )

            if has_mpx:
                # Test write and read for bnd0.
                new_value_w = "{0x01 0x02 0x03 0x04 0x05 0x06 0x07 0x08 0x09 0x0a 0x0b 0x0c 0x0d 0x0e 0x0f 0x10}"
                self.runCmd("register write bnd0 '" + new_value_w + "'")
                new_value_r = "{0x0807060504030201 0x100f0e0d0c0b0a09}"
                self.expect("register read bnd0", substrs=["bnd0 = ", new_value_r])
                self.expect("expr $bnd0", substrs=["vector_type"])

                # Test write and for bndstatus.
                new_value = "{0x01 0x02 0x03 0x04 0x05 0x06 0x07 0x08}"
                self.write_and_read(currentFrame, "bndstatus", new_value)
                self.expect("expr $bndstatus", substrs=["vector_type"])
            elif has_mpx_regs:
                self.expect("register read bnd0", substrs=["error: unavailable"])
            else:
                self.expect(
                    "register read bnd0",
                    substrs=["Invalid register name 'bnd0'"],
                    error=True,
                )

    def convenience_registers(self):
        """Test convenience registers."""
        self.common_setup()

        # The command "register read -a" does output a derived register like
        # eax...
        self.expect("register read -a", matching=True, substrs=["eax"])

        # ...however, the vanilla "register read" command should not output derived registers like eax.
        self.expect("register read", matching=False, substrs=["eax"])

        # Test reading of rax and eax.
        self.expect("register read rax eax", substrs=["rax = 0x", "eax = 0x"])

        # Now write rax with a unique bit pattern and test that eax indeed
        # represents the lower half of rax.
        self.runCmd("register write rax 0x1234567887654321")
        self.expect("register read rax", substrs=["0x1234567887654321"])

    def convenience_registers_with_process_attach(self, test_16bit_regs):
        """Test convenience registers after a 'process attach'."""
        exe = self.getBuildArtifact("a.out")

        # Spawn a new process
        pid = self.spawnSubprocess(exe, ["wait_for_attach"]).pid

        if self.TraceOn():
            print("pid of spawned process: %d" % pid)

        self.runCmd("process attach -p %d" % pid)

        # Check that "register read eax" works.
        self.runCmd("register read eax")

        if self.getArchitecture() in ["amd64", "x86_64"]:
            self.expect("expr -- ($rax & 0xffffffff) == $eax", substrs=["true"])

        if test_16bit_regs:
            self.expect("expr -- $ax == (($ah << 8) | $al)", substrs=["true"])

    @skipIfiOSSimulator
    @skipIf(archs=no_match(["amd64", "arm$", "i386", "x86_64"]))
    def test_invalid_invocation(self):
        self.build()
        self.common_setup()

        self.expect(
            "register read -a arg",
            error=True,
            substrs=[
                "the --all option can't be used when registers names are supplied as arguments"
            ],
        )

        self.expect(
            "register read --set 0 r",
            error=True,
            substrs=[
                "the --set <set> option can't be used when registers names are supplied as arguments"
            ],
        )

        self.expect(
            "register write a",
            error=True,
            substrs=["register write takes exactly 2 arguments: <reg-name> <value>"],
        )
        self.expect(
            "register write a b c",
            error=True,
            substrs=["register write takes exactly 2 arguments: <reg-name> <value>"],
        )

    @skipIfiOSSimulator
    @skipIf(archs=no_match(["amd64", "arm$", "i386", "x86_64"]))
    def test_write_unknown_register(self):
        self.build()
        self.common_setup()

        self.expect(
            "register write blub 1",
            error=True,
            substrs=["error: Register not found for 'blub'."],
        )

    def test_info_unknown_register(self):
        self.build()
        self.common_setup()

        self.expect(
            "register info blub",
            error=True,
            substrs=["error: No register found with name 'blub'."],
        )

    def test_info_many_registers(self):
        self.build()
        self.common_setup()

        # Only 1 register allowed at this time.
        self.expect(
            "register info abc def",
            error=True,
            substrs=["error: register info takes exactly 1 argument"],
        )

    @skipIf(archs=no_match(["aarch64"]))
    def test_info_register(self):
        # The behaviour of this command is generic but the specific registers
        # are not, so this is written for AArch64 only.
        # Text alignment and ordering are checked in the DumpRegisterInfo and
        # RegisterFlags unit tests.
        self.build()
        self.common_setup()

        # Standard register. Doesn't invalidate anything, doesn't have an alias.
        self.expect(
            "register info x1",
            substrs=[
                "Name: x1",
                "Size: 8 bytes (64 bits)",
                "In sets: General Purpose Registers",
            ],
        )
        self.expect(
            "register info x1", substrs=["Invalidates:", "Name: x1 ("], matching=False
        )

        # These registers invalidate others as they are subsets of those registers.
        self.expect("register info w1", substrs=["Invalidates: x1"])
        self.expect("register info s0", substrs=["Invalidates: v0, d0"])

        # This has an alternative name according to the ABI.
        self.expect("register info x30", substrs=["Name: lr (x30)"])

    @skipIfXmlSupportMissing
    @skipUnlessPlatform(["linux", "freebsd"])
    @skipIf(archs=no_match(["aarch64"]))
    def test_register_read_fields(self):
        """Test that when debugging a live process, we see the fields of certain
        registers."""
        self.build()
        self.common_setup()

        # N/Z/C/V bits will always be present, so check only for those.
        self.expect(
            "register read cpsr",
            patterns=[r"= \(N = [0|1], Z = [0|1], C = [0|1], V = [0|1]"],
        )
        self.expect(
            "register read fpsr", patterns=[r"= \(QC = [0|1], IDC = [0|1], IXC = [0|1]"]
        )
        # AHP/DN/FZ always present, others may vary.
        self.expect(
            "register read fpcr", patterns=[r"= \(AHP = [0|1], DN = [0|1], FZ = [0|1]"]
        )

        # Should get enumerator descriptions for RMode.
        self.expect(
            "register info fpcr",
            substrs=["RMode: 0 = RN, 1 = RP, 2 = RM, 3 = RZ"],
        )

    @skipUnlessPlatform(["linux"])
    @skipIf(archs=no_match(["x86_64"]))
    def test_fs_gs_base(self):
        """
        Tests fs_base register can be read and equals to pthread_self() return value
        and gs_base register equals zero.
        """
        self.build()
        target = self.createTestTarget()
        # Launch the process and stop.
        self.expect("run", PROCESS_STOPPED, substrs=["stopped"])

        process = target.GetProcess()

        thread = process.GetThreadAtIndex(0)
        self.assertTrue(thread.IsValid(), "current thread is valid")

        current_frame = thread.GetFrameAtIndex(0)
        self.assertTrue(current_frame.IsValid(), "current frame is valid")

        reg_fs_base = current_frame.FindRegister("fs_base")
        self.assertTrue(reg_fs_base.IsValid(), "fs_base is not available")
        reg_gs_base = current_frame.FindRegister("gs_base")
        self.assertTrue(reg_gs_base.IsValid(), "gs_base is not available")
        self.assertEqual(reg_gs_base.GetValueAsSigned(-1), 0, f"gs_base should be zero")

        # Evaluate pthread_self() and compare against fs_base register read.
        pthread_self_code = "(uint64_t)pthread_self()"
        pthread_self_val = current_frame.EvaluateExpression(pthread_self_code)
        self.assertTrue(
            pthread_self_val.IsValid(), f"{pthread_self_code} evaluation has failed"
        )
        self.assertNotEqual(
            reg_fs_base.GetValueAsSigned(-1), -1, f"fs_base returned -1 which is wrong"
        )

        self.assertEqual(
            reg_fs_base.GetValueAsUnsigned(0),
            pthread_self_val.GetValueAsUnsigned(0),
            "fs_base does not equal to pthread_self() value.",
        )

    def test_process_must_be_stopped(self):
        """Check that all register commands error when the process is not stopped."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        pid = self.spawnSubprocess(exe, ["wait_for_attach"]).pid
        # Async so we can enter commands while the process is running.
        self.setAsync(True)
        self.runCmd("process attach --continue -p %d" % pid)

        err_msg = "Command requires a process which is currently stopped."
        self.expect("register read pc", substrs=[err_msg], error=True)
        self.expect("register write pc 0", substrs=[err_msg], error=True)
        self.expect("register info pc", substrs=[err_msg], error=True)



"""
Test lldb-dap IO handling.
"""

import sys

from lldbsuite.test.decorators import *
import lldbdap_testcase
import dap_server

EXIT_FAILURE = 1
EXIT_SUCCESS = 0


class TestDAP_io(lldbdap_testcase.DAPTestCaseBase):
    def launch(self):
        log_file_path = self.getBuildArtifact("dap.log")
        process, _ = dap_server.DebugAdapterServer.launch(
            executable=self.lldbDAPExec, log_file=log_file_path
        )

        def cleanup():
            # If the process is still alive, terminate it.
            if process.poll() is None:
                process.terminate()
                process.wait()
            stdout_data = process.stdout.read().decode()
            print("========= STDOUT =========", file=sys.stderr)
            print(stdout_data, file=sys.stderr)
            print("========= END =========", file=sys.stderr)
            print("========= DEBUG ADAPTER PROTOCOL LOGS =========", file=sys.stderr)
            with open(log_file_path, "r") as file:
                print(file.read(), file=sys.stderr)
            print("========= END =========", file=sys.stderr)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        return process

    def test_eof_immediately(self):
        """
        lldb-dap handles EOF without any other input.
        """
        process = self.launch()
        process.stdin.close()
        self.assertEqual(process.wait(timeout=self.DEFAULT_TIMEOUT), EXIT_SUCCESS)

    def test_invalid_header(self):
        """
        lldb-dap returns a failure exit code when the input stream is closed
        with a malformed request header.
        """
        process = self.launch()
        process.stdin.write(b"not the correct message header")
        process.stdin.close()
        self.assertEqual(process.wait(timeout=self.DEFAULT_TIMEOUT), EXIT_FAILURE)

    def test_partial_header(self):
        """
        lldb-dap returns a failure exit code when the input stream is closed
        with an incomplete message header is in the message buffer.
        """
        process = self.launch()
        process.stdin.write(b"Content-Length: ")
        process.stdin.close()
        self.assertEqual(process.wait(timeout=self.DEFAULT_TIMEOUT), EXIT_FAILURE)

    def test_incorrect_content_length(self):
        """
        lldb-dap returns a failure exit code when reading malformed content
        length headers.
        """
        process = self.launch()
        process.stdin.write(b"Content-Length: abc")
        process.stdin.close()
        self.assertEqual(process.wait(timeout=self.DEFAULT_TIMEOUT), EXIT_FAILURE)

    def test_partial_content_length(self):
        """
        lldb-dap returns a failure exit code when the input stream is closed
        with a partial message in the message buffer.
        """
        process = self.launch()
        process.stdin.write(b"Content-Length: 10\r\n\r\n{")
        process.stdin.close()
        self.assertEqual(process.wait(timeout=self.DEFAULT_TIMEOUT), EXIT_FAILURE)



"""
Test that LLDB correctly reads and writes and restores AArch64 SIMD registers
in SVE, streaming SVE and normal SIMD modes.

There are a few operating modes and we use different strategies for each:
* Without SVE, in SIMD mode - read the SIMD regset.
* With SVE, but SVE is inactive - read the SVE regset, but get SIMD data from it.
* With SVE, SVE is active - read the SVE regset, use the bottom 128 bits of the
  Z registers.
* With streaming SVE active - read the SSVE regset, use the bottom 128 bits of
  the Z registers.

This text excercise most of those.
"""

from enum import Enum
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class Mode(Enum):
    SIMD = 0
    SVE = 1
    SSVE = 2


class SVESIMDRegistersTestCase(TestBase):
    def get_build_flags(self, mode):
        cflags = "-march=armv8-a+sve"
        if mode == Mode.SSVE:
            cflags += " -DSSVE"
        elif mode == Mode.SVE:
            cflags += " -DSVE"
        # else we want SIMD mode, which processes start up in already.

        return {"CFLAGS_EXTRAS": cflags}

    def skip_if_needed(self, mode):
        if (mode == Mode.SVE) and not self.isAArch64SVE():
            self.skipTest("SVE registers must be supported.")

        if (mode == Mode.SSVE) and not self.isAArch64SMEFA64():
            self.skipTest(
                "SSVE registers must be supported and the smefa64 "
                "extension must be present."
            )

    def make_simd_value(self, n):
        pad = " ".join(["0x00"] * 7)
        return "{{0x{:02x} {} 0x{:02x} {}}}".format(n, pad, n, pad)

    def check_simd_values(self, value_offset):
        # These are 128 bit registers, so getting them from the API as unsigned
        # values doesn't work. Check the command output instead.
        for i in range(32):
            self.expect(
                "register read v{}".format(i),
                substrs=[self.make_simd_value(i + value_offset)],
            )

    def sve_simd_registers_impl(self, mode):
        self.skip_if_needed(mode)

        self.build(dictionary=self.get_build_flags(mode))
        self.line = line_number("main.c", "// Set a break point here.")

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "thread backtrace",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        self.check_simd_values(0)
        self.runCmd("expression write_simd_regs(1)")
        self.check_simd_values(0)

        # Write a new set of values. The kernel will move the program back to
        # non-streaming mode here.
        for i in range(32):
            self.runCmd(
                'register write v{} "{}"'.format(i, self.make_simd_value(i + 1))
            )

        # Should be visible within lldb.
        self.check_simd_values(1)

        # The program should agree with lldb.
        self.expect("continue", substrs=["exited with status = 0"])

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_simd_registers_sve(self):
        """Test read/write of SIMD registers when in SVE mode."""
        self.sve_simd_registers_impl(Mode.SVE)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_simd_registers_ssve(self):
        """Test read/write of SIMD registers when in SSVE mode."""
        self.sve_simd_registers_impl(Mode.SSVE)

    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_simd_registers_simd(self):
        """Test read/write of SIMD registers when in SIMD mode."""
        self.sve_simd_registers_impl(Mode.SIMD)



"""
Test that the user can input a format but it will not prevail over summary format's choices.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class FrameFormatSmallStructTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break at.
        self.line = line_number("main.cpp", "// Set break point at this line.")

    def test_with_run_command(self):
        """Test that the user can input a format but it will not prevail over summary format's choices."""
        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        self.expect("thread list", substrs=["addPair(p=(x = 3, y = -3))"])



"""
Test that LLDB correctly reads, writes and restores the MTE control register on
AArch64 Linux.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class MTECtrlRegisterTestCase(TestBase):
    @no_debug_info_test
    @skipIf(archs=no_match(["aarch64"]))
    @skipIf(oslist=no_match(["linux"]))
    def test_mte_ctrl_register(self):
        if not self.isAArch64MTE():
            self.skipTest("Target must support MTE.")

        self.build()
        self.line = line_number("main.c", "// Set a break point here.")

        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line, num_expected_locations=1
        )
        self.runCmd("run", RUN_SUCCEEDED)

        self.expect(
            "process status",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stop reason = breakpoint 1."],
        )

        has_store_only = self.isAArch64MTEStoreOnly()

        def check_mte_ctrl(async_err, sync_err, store_only):
            # Bit 0 = tagged addressing enabled
            # Bit 1 = synchronous faults
            # Bit 2 = asynchronous faults
            # Bit 19 = store only checking mode
            value = "0x{:016x}".format(
                (store_only << 19) | (async_err << 2) | (sync_err << 1) | 1
            )
            expected = [value]

            if self.hasXMLSupport():
                fields = "("
                if has_store_only:
                    fields += f"STORE_ONLY = {store_only}, "

                tfc_modes = ["NONE", "SYNC", "ASYNC", "ASYMM"]
                fields += f"TAGS = 0, TCF = TCF_{tfc_modes[async_err << 1 | sync_err]}, TAGGED_ADDR_ENABLE = 1)"

                expected.append(fields)

            self.expect("register read mte_ctrl", substrs=expected)

        # We start enabled with synchronous faults.
        check_mte_ctrl(0, 1, 0)
        # Change to asynchronous faults.
        self.runCmd("register write mte_ctrl 5")
        check_mte_ctrl(1, 0, 0)
        # This would return to synchronous faults if we did not restore the
        # previous value.
        self.expect("expression setup_mte()", substrs=["= 0"])
        check_mte_ctrl(1, 0, 0)

        # Store only checking requires FEAT_MTE_STORE_ONLY.
        if has_store_only:
            self.runCmd(f"register write mte_ctrl {1 | (1 << 19)}")
            check_mte_ctrl(0, 0, 1)



"""
Test that variables of type short are displayed correctly.
"""

import AbstractBase

from lldbsuite.test.decorators import *


class ShortTypeTestCase(AbstractBase.GenericTester):
    def test_short_type(self):
        """Test that short-type variables are displayed correctly."""
        self.build_and_run("short.cpp", ["short"])

    @skipUnlessDarwin
    def test_short_type_from_block(self):
        """Test that short-type variables are displayed correctly from a block."""
        self.build_and_run("short.cpp", ["short"], bc=True)

    def test_unsigned_short_type(self):
        """Test that 'unsigned_short'-type variables are displayed correctly."""
        self.build_and_run("unsigned_short.cpp", ["unsigned", "short"])

    @skipUnlessDarwin
    def test_unsigned_short_type_from_block(self):
        """Test that 'unsigned short'-type variables are displayed correctly from a block."""
        self.build_and_run("unsigned_short.cpp", ["unsigned", "short"], bc=True)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.m"))
        self.expect(
            "vo pair",
            substrs=["error:", "not a pointer type", "(Pair) pair = (f = 2, e = 3)"],
        )
        self.expect(
            "expr -O -- pair",
            substrs=["error:", "not a pointer type", "(Pair)  (f = 2, e = 3)"],
        )



# RUN: %PYTHON %s 2>&1 | FileCheck %s

from mlir.ir import *
from mlir.dialects.irdl import *
import sys


def run(f):
    print("\nTEST:", f.__name__, file=sys.stderr)
    f()


# CHECK: TEST: testIRDL
@run
def testIRDL():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            irdl_test = dialect("irdl_test")
            with InsertionPoint(irdl_test.body):
                op = operation_("test_op")
                with InsertionPoint(op.body):
                    f32 = is_(TypeAttr.get(F32Type.get()))
                    operands_([f32], ["input"], [Variadicity.single])
                type1 = type_("type1")
                with InsertionPoint(type1.body):
                    f32 = is_(TypeAttr.get(F32Type.get()))
                    parameters([f32], ["val"])
                attr1 = attribute("attr1")
                with InsertionPoint(attr1.body):
                    test = is_(StringAttr.get("test"))
                    parameters([test], ["val"])

        # CHECK: module {
        # CHECK:   irdl.dialect @irdl_test {
        # CHECK:     irdl.operation @test_op {
        # CHECK:       %0 = irdl.is f32
        # CHECK:       irdl.operands(input: %0)
        # CHECK:     }
        # CHECK:     irdl.type @type1 {
        # CHECK:       %0 = irdl.is f32
        # CHECK:       irdl.parameters(val: %0)
        # CHECK:     }
        # CHECK:     irdl.attribute @attr1 {
        # CHECK:       %0 = irdl.is "test"
        # CHECK:       irdl.parameters(val: %0)
        # CHECK:     }
        # CHECK:   }
        # CHECK: }
        module.operation.verify()
        module.dump()

        load_dialects(module)

        m = Module.parse(
            """
          module {
            %a = arith.constant 1.0 : f32
            "irdl_test.test_op"(%a) : (f32) -> ()
          }
        """
        )
        # CHECK: module {
        # CHECK:   "irdl_test.test_op"(%cst) : (f32) -> ()
        # CHECK: }
        m.dump()



# RUN: %PYTHON %s | FileCheck %s
# This is just a smoke test that the dialect is functional.

from mlir.ir import *
from mlir.dialects import nvvm
from mlir.dialects import llvm
from mlir.dialects import func
import mlir.extras.types as T
from mlir.dialects import arith


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()

        print(module)
        module.operation.verify()
    return f


# CHECK-LABEL: testSmoke
@constructAndPrintInModule
def testSmoke():
    i64 = IntegerType.get_signless(64)
    mat64f32_t = Type.parse(
        "!llvm.struct<(f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32)>"
    )
    shape_attr = Attribute.parse("#nvvm.shape<m = 64, n = 32, k = 16>")

    # CHECK-LABEL: func @wgmma_f32_f16_f16(%arg0: i64, %arg1: i64)
    @func.FuncOp.from_py_func(i64, i64)
    def wgmma_f32_f16_f16(desc_a, desc_b):
        # CHECK: nvvm.cp.async.wait.group 5
        nvvm.CpAsyncWaitGroupOp(5)
        # CHECK: %0 = llvm.mlir.undef : [[MAT_T:.*]]
        result = llvm.UndefOp(mat64f32_t)
        # CHECK: %1 = nvvm.wgmma.mma_async %arg0, %arg1, %0, <m = 64, n = 32, k = 16>, D[<f32>, <zero>], A[<f16>, <neg>, <col>], B[<f16>, <neg>, <col>] : [[MAT_T]] -> [[MAT_T]]
        result1 = nvvm.WgmmaMmaAsyncOp(
            results_=mat64f32_t,
            inouts=result,
            descriptorA=desc_a,
            descriptorB=desc_b,
            shape=shape_attr,
            typeA=nvvm.WGMMATypes.f16,
            typeB=nvvm.WGMMATypes.f16,
            typeD=nvvm.WGMMATypes.f32,
            scaleD=nvvm.WGMMAScaleOut.zero,
            scaleA=nvvm.WGMMAScaleIn.neg,
            scaleB=nvvm.WGMMAScaleIn.neg,
            layoutA=nvvm.MMALayout.col,
            layoutB=nvvm.MMALayout.col,
        )


# CHECK-LABEL: TEST: test_inline_ptx
# CHECK-LABEL: func.func @my_inline_ptx(
# CHECK-SAME: %[[arg0:[a-zA-Z0-9_]+]]: f32, %[[arg1:[a-zA-Z0-9_]+]]: f32, %[[arg2:[a-zA-Z0-9_]+]]: i32, %[[arg3:[a-zA-Z0-9_]+]]: i32)
# CHECK: %[[S0:.+]]:2 = nvvm.inline_ptx
# CHECK-SAME: ro(%[[arg0]], %[[arg1]] : f32, f32) rw(%[[arg2]], %[[arg3]] : i32, i32) -> f32, f32
# CHECK: %[[S1:.+]] = arith.addf %[[arg0]], %[[arg1]] : f32
# CHECK: %[[S2:.+]] = arith.addi %[[arg2]], %[[arg3]] : i32
# CHECK: %[[S3:.+]] = arith.addf %[[S0]]#0, %[[S0]]#1 : f32


@constructAndPrintInModule
def test_inline_ptx():
    i32 = T.i32()
    f32 = T.f32()

    @func.FuncOp.from_py_func(f32, f32, i32, i32)
    def my_inline_ptx(a, b, c, d):
        ptx = r"""
            {
                .reg .pred p;
                setp.ge.s32   p,      {$r0}, {$r1};
                selp.s32      {$r0},  {$r0}, {$r1}, p;
                selp.s32      {$r1},  {$r0}, {$r1}, p;
                selp.s32      {$rw0}, {$r0}, {$r1}, p;
                selp.s32      {$rw1}, {$r0}, {$r1}, p;
            }
            """
        wo0, wo1 = nvvm.inline_ptx(
            read_only_args=[a, b],
            read_write_args=[c, d],
            write_only_args=[f32, f32],
            ptx_code=ptx,
        )
        arith.addf(a, b)
        arith.addi(c, d)
        arith.addf(wo0, wo1)


@constructAndPrintInModule
def test_barriers():
    i32 = T.i32()
    f32 = T.f32()

    @func.FuncOp.from_py_func(i32, i32, f32)
    def barriers(mask, vi32, vf32):
        c0 = arith.constant(T.i32(), 0)
        cffff = arith.constant(T.i32(), 0xFFFF)
        res = nvvm.barrier(
            res=i32,
            barrier_id=c0,
            number_of_threads=cffff,
        )

        for reduction in (
            nvvm.BarrierReduction.AND,
            nvvm.BarrierReduction.OR,
            nvvm.BarrierReduction.POPC,
        ):
            res = nvvm.barrier(
                res=i32,
                reduction_op=reduction,
                reduction_predicate=res,
            )

        nvvm.barrier0()
        nvvm.bar_warp_sync(mask)
        nvvm.cluster_arrive()
        nvvm.cluster_arrive(aligned=True)
        nvvm.cluster_arrive_relaxed()
        nvvm.cluster_arrive_relaxed(aligned=True)
        nvvm.cluster_wait()
        nvvm.cluster_wait(aligned=True)
        nvvm.fence_mbarrier_init()
        nvvm.bar_warp_sync(mask)
        return res


# CHECK-LABEL:   func.func @barriers(
# CHECK:           %[[ARG0:.*]]: i32, %[[ARG1:.*]]: i32, %[[ARG2:.*]]: f32) -> i32 {
# CHECK:           %[[CONSTANT_0:.*]] = arith.constant 0 : i32
# CHECK:           %[[CONSTANT_1:.*]] = arith.constant 65535 : i32
# CHECK:           %[[BARRIER_0:.*]] = nvvm.barrier id = %[[CONSTANT_0]] number_of_threads = %[[CONSTANT_1]] -> i32
# CHECK:           %[[BARRIER_1:.*]] = nvvm.barrier #nvvm.reduction<and> %[[BARRIER_0]] -> i32
# CHECK:           %[[BARRIER_2:.*]] = nvvm.barrier #nvvm.reduction<or> %[[BARRIER_1]] -> i32
# CHECK:           %[[BARRIER_3:.*]] = nvvm.barrier #nvvm.reduction<popc> %[[BARRIER_2]] -> i32
# CHECK:           nvvm.barrier0
# CHECK:           nvvm.bar.warp.sync %[[ARG0]] : i32
# CHECK:           nvvm.cluster.arrive
# CHECK:           nvvm.cluster.arrive {aligned}
# CHECK:           nvvm.cluster.arrive.relaxed
# CHECK:           nvvm.cluster.arrive.relaxed {aligned}
# CHECK:           nvvm.cluster.wait
# CHECK:           nvvm.cluster.wait {aligned}
# CHECK:           nvvm.fence.mbarrier.init
# CHECK:           nvvm.bar.warp.sync %[[ARG0]] : i32
# CHECK:           return %[[BARRIER_3]] : i32
# CHECK:         }


@constructAndPrintInModule
def test_reductions():
    i32 = T.i32()
    f32 = T.f32()

    @func.FuncOp.from_py_func(i32, i32, f32)
    def reductions(mask, vi32, vf32):
        for abs in (True, False):
            for nan in (True, False):
                for kind in (
                    nvvm.ReduxKind.AND,
                    nvvm.ReduxKind.MAX,
                    nvvm.ReduxKind.MIN,
                    nvvm.ReduxKind.OR,
                    nvvm.ReduxKind.UMAX,
                    nvvm.ReduxKind.UMIN,
                    nvvm.ReduxKind.XOR,
                ):
                    nvvm.redux_sync(i32, vi32, kind, vi32)

                for kind in (
                    nvvm.ReduxKind.FMIN,
                    nvvm.ReduxKind.FMAX,
                ):
                    nvvm.redux_sync(f32, vf32, kind, vi32, abs=abs, nan=nan)


# CHECK-LABEL:   func.func @reductions(
# CHECK:           %[[ARG0:.*]]: i32, %[[ARG1:.*]]: i32, %[[ARG2:.*]]: f32) {
# CHECK:           %[[REDUX_0:.*]] = nvvm.redux.sync  and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_1:.*]] = nvvm.redux.sync  max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_2:.*]] = nvvm.redux.sync  min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_3:.*]] = nvvm.redux.sync  or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_4:.*]] = nvvm.redux.sync  umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_5:.*]] = nvvm.redux.sync  umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_6:.*]] = nvvm.redux.sync  xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_7:.*]] = nvvm.redux.sync  fmin %[[ARG2]], %[[ARG1]] {abs = true, nan = true} : f32 -> f32
# CHECK:           %[[REDUX_8:.*]] = nvvm.redux.sync  fmax %[[ARG2]], %[[ARG1]] {abs = true, nan = true} : f32 -> f32
# CHECK:           %[[REDUX_9:.*]] = nvvm.redux.sync  and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_10:.*]] = nvvm.redux.sync  max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_11:.*]] = nvvm.redux.sync  min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_12:.*]] = nvvm.redux.sync  or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_13:.*]] = nvvm.redux.sync  umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_14:.*]] = nvvm.redux.sync  umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_15:.*]] = nvvm.redux.sync  xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_16:.*]] = nvvm.redux.sync  fmin %[[ARG2]], %[[ARG1]] {abs = true} : f32 -> f32
# CHECK:           %[[REDUX_17:.*]] = nvvm.redux.sync  fmax %[[ARG2]], %[[ARG1]] {abs = true} : f32 -> f32
# CHECK:           %[[REDUX_18:.*]] = nvvm.redux.sync  and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_19:.*]] = nvvm.redux.sync  max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_20:.*]] = nvvm.redux.sync  min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_21:.*]] = nvvm.redux.sync  or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_22:.*]] = nvvm.redux.sync  umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_23:.*]] = nvvm.redux.sync  umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_24:.*]] = nvvm.redux.sync  xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_25:.*]] = nvvm.redux.sync  fmin %[[ARG2]], %[[ARG1]] {nan = true} : f32 -> f32
# CHECK:           %[[REDUX_26:.*]] = nvvm.redux.sync  fmax %[[ARG2]], %[[ARG1]] {nan = true} : f32 -> f32
# CHECK:           %[[REDUX_27:.*]] = nvvm.redux.sync  and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_28:.*]] = nvvm.redux.sync  max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_29:.*]] = nvvm.redux.sync  min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_30:.*]] = nvvm.redux.sync  or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_31:.*]] = nvvm.redux.sync  umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_32:.*]] = nvvm.redux.sync  umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_33:.*]] = nvvm.redux.sync  xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_34:.*]] = nvvm.redux.sync  fmin %[[ARG2]], %[[ARG1]] : f32 -> f32
# CHECK:           %[[REDUX_35:.*]] = nvvm.redux.sync  fmax %[[ARG2]], %[[ARG1]] : f32 -> f32
# CHECK:           return
# CHECK:         }



""" Check that unavailable registers are reported when reading register sets."""

from textwrap import dedent
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class MyResponder(MockGDBServerResponder):
    def readRegisters(self):
        return "E01"

    def readRegister(self, regnum):
        if regnum in [0, 1, 2]:
            return "E01"
        return "5555555555555555"

    def qHostInfo(self):
        # The triple is hex encoded ASCII "x86_64-linux-gnu".
        return "triple:7838365F36342D6C696E75782D676E75;"


class TestRegistersUnavailable(GDBRemoteTestBase):
    @skipIfRemote
    # So that we have multiple register sets.
    @skipIfLLVMTargetMissing("X86")
    def test_unavailable_registers(self):
        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")

        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets process")
            self.addTearDownHook(
                lambda: self.runCmd("log disable gdb-remote packets process")
            )

        process = self.connect(target)
        lldbutil.expect_state_changes(
            self, self.dbg.GetListener(), process, [lldb.eStateStopped]
        )

        # We are using a long regex pattern here to check that the indentation
        # is correct when you have multiple register sets and they all have
        # some missing registers.
        self.expect(
            "register read --all",
            patterns=[
                "(?sm)^general purpose registers:\n"
                "^\\s+rdx = 0x5555555555555555\n"
                ".*"
                "^3 registers were unavailable.\n"
                "\n"
                "^supplementary registers:\n"
                "^\\s+edx = 0x55555555\n"
                ".*"
                "^12 registers were unavailable."
            ],
        )



"""
Test DIL pointer arithmetic.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestFrameVarDILExprPointerArithmetic(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_pointer_arithmetic(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.experimental.use-DIL true")

        self.expect_var_path("+array", type="int *")
        self.expect_var_path("+array_ref", type="int *")
        self.expect_var_path("+p_int0", type="int *")
        self.expect(
            "frame var -- '-p_int0'",
            error=True,
            substrs=["invalid argument type 'int *' to unary expression"],
        )



"""
Test lldb-dap setBreakpoints request
"""

from dap_server import Source
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase
import os


class TestDAP_breakpointEvents(lldbdap_testcase.DAPTestCaseBase):
    @skipUnlessDarwin
    def test_breakpoint_events(self):
        """
        This test sets a breakpoint in a shared library and runs and stops
        at the entry point of a program. When we stop at the entry point,
        the shared library won't be loaded yet. At this point the
        breakpoint should set itself, but not be verified because no
        locations are resolved. We will then continue and expect to get a
        breakpoint event that informs us that the breakpoint in the shared
        library is "changed" and the correct line number should be
        supplied. We also set a breakpoint using a LLDB command using the
        "preRunCommands" when launching our program. Any breakpoints set via
        the command interpreter should not be have breakpoint events sent
        back to VS Code as the UI isn't able to add new breakpoints to
        their UI. Code has been added that tags breakpoints set from VS Code
        DAP packets so we know the IDE knows about them. If VS Code is ever
        able to register breakpoints that aren't initially set in the GUI,
        then we will need to revise this.
        """
        main_source_basename = "main.cpp"
        main_source_path = os.path.join(os.getcwd(), main_source_basename)
        foo_source_basename = "foo.cpp"
        foo_source_path = os.path.join(os.getcwd(), foo_source_basename)
        main_bp_line = line_number("main.cpp", "main breakpoint 1")
        foo_bp1_line = line_number("foo.cpp", "foo breakpoint 1")
        foo_bp2_line = line_number("foo.cpp", "foo breakpoint 2")

        # Visual Studio Code Debug Adapters have no way to specify the file
        # without launching or attaching to a process, so we must start a
        # process in order to be able to set breakpoints.
        program = self.getBuildArtifact("a.out")

        # Set a breakpoint after creating the target by running a command line
        # command. It will eventually resolve and cause a breakpoint changed
        # event to be sent to lldb-dap. We want to make sure we don't send a
        # breakpoint any breakpoints that were set from the command line.
        # Breakpoints that are set via the VS code DAP packets will be
        # registered and marked with a special keyword to ensure we deliver
        # breakpoint events for these breakpoints but not for ones that are not
        # set via the command interpreter.
        bp_command = "breakpoint set --file foo.cpp --line %u" % (foo_bp2_line)
        self.build_and_launch(program, preRunCommands=[bp_command])
        main_bp_id = 0
        foo_bp_id = 0
        # Set breakpoints and verify that they got set correctly
        dap_breakpoint_ids = []
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=main_source_path), [main_bp_line]
        )
        self.assertTrue(response["success"])
        breakpoints = response["body"]["breakpoints"]
        for breakpoint in breakpoints:
            main_bp_id = breakpoint["id"]
            dap_breakpoint_ids.append("%i" % (main_bp_id))
            self.assertTrue(
                breakpoint["verified"], "expect main breakpoint to be verified"
            )

        response = self.dap_server.request_setBreakpoints(
            Source.build(path=foo_source_path), [foo_bp1_line]
        )
        self.assertTrue(response["success"])
        breakpoints = response["body"]["breakpoints"]
        for breakpoint in breakpoints:
            foo_bp_id = breakpoint["id"]
            dap_breakpoint_ids.append("%i" % (foo_bp_id))
            self.assertFalse(
                breakpoint["verified"], "expect foo breakpoint to not be verified"
            )

        # Flush the breakpoint events.
        self.dap_server.wait_for_breakpoint_events()

        # Continue to the breakpoint
        self.continue_to_breakpoints(dap_breakpoint_ids)

        verified_breakpoint_ids = []
        unverified_breakpoint_ids = []
        for breakpoint_event in self.dap_server.wait_for_breakpoint_events():
            breakpoint = breakpoint_event["body"]["breakpoint"]
            id = breakpoint["id"]
            if breakpoint["verified"]:
                verified_breakpoint_ids.append(id)
            else:
                unverified_breakpoint_ids.append(id)

        self.assertIn(main_bp_id, unverified_breakpoint_ids)
        self.assertIn(foo_bp_id, unverified_breakpoint_ids)

        self.assertIn(main_bp_id, verified_breakpoint_ids)
        self.assertIn(foo_bp_id, verified_breakpoint_ids)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.m"))
        self.expect(
            "expr -O -- bad", substrs=["error:", "expression interrupted", "(Bad *) 0x"]
        )
        self.expect(
            "dwim-print -O -- bad",
            substrs=["error:", "expression interrupted", "_lookHere = NO"],
        )



"""
Test frame providers for scripted frame provider functionality.

These providers demonstrate various merge strategies:
- Replace: Replace entire stack
- Prepend: Add frames before real stack
- Append: Add frames after real stack

It also shows the ability to mix a dictionary, a ScriptedFrame or an SBFrame
index to create stackframes
"""

import os
import lldb
from lldb.plugins.scripted_process import ScriptedFrame
from lldb.plugins.scripted_frame_provider import ScriptedFrameProvider


class ReplaceFrameProvider(ScriptedFrameProvider):
    """Replace entire stack with custom frames."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)
        self.frames = [
            {
                "idx": 0,
                "pc": 0x1000,
            },
            0,
            {
                "idx": 2,
                "pc": 0x3000,
            },
        ]

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Replace entire stack with 3 custom frames"

    def get_frame_at_index(self, index):
        if index >= len(self.frames):
            return None
        return self.frames[index]


class PrependFrameProvider(ScriptedFrameProvider):
    """Prepend synthetic frames before real stack."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Prepend 2 synthetic frames before real stack"

    def get_frame_at_index(self, index):
        if index == 0:
            return {"pc": 0x9000}
        elif index == 1:
            return {"pc": 0xA000}
        elif index - 2 < len(self.input_frames):
            return index - 2  # Return real frame index.
        return None


class AppendFrameProvider(ScriptedFrameProvider):
    """Append synthetic frames after real stack."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Append 1 synthetic frame after real stack"

    def get_frame_at_index(self, index):
        if index < len(self.input_frames):
            return index  # Return real frame index.
        elif index == len(self.input_frames):
            return {
                "idx": 1,
                "pc": 0x10,
            }
        return None


class CustomScriptedFrame(ScriptedFrame):
    """Custom scripted frame with full control over frame behavior."""

    def __init__(self, thread, idx, pc, function_name):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.pc = pc
        self.function_name = function_name

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """Return the program counter."""
        return self.pc

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def is_artificial(self):
        """Mark as artificial frame."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context for this test."""
        return None


class ScriptedFrameObjectProvider(ScriptedFrameProvider):
    """Provider that returns ScriptedFrame objects instead of dictionaries."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider returning custom ScriptedFrame objects"

    def get_frame_at_index(self, index):
        """Return ScriptedFrame objects or dictionaries based on index."""
        if index == 0:
            return CustomScriptedFrame(
                self.thread, 0, 0x5000, "custom_scripted_frame_0"
            )
        elif index == 1:
            return {"pc": 0x6000}
        elif index == 2:
            return CustomScriptedFrame(
                self.thread, 2, 0x7000, "custom_scripted_frame_2"
            )
        elif index == 3:
            return len(self.input_frames) - 2  # Real frame index.
        elif index == 4:
            return len(self.input_frames) - 1  # Real frame index.
        return None


class ThreadFilterFrameProvider(ScriptedFrameProvider):
    """Provider that only applies to thread with ID 1."""

    @staticmethod
    def applies_to_thread(thread):
        """Only apply to thread with index ID 1."""
        return thread.GetIndexID() == 1

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that only applies to thread ID 1"

    def get_frame_at_index(self, index):
        """Return a single synthetic frame."""
        if index == 0:
            return {"pc": 0xFFFF}
        return None


class CircularDependencyTestProvider(ScriptedFrameProvider):
    """
    Provider that tests the circular dependency fix.

    This provider accesses input_frames during __init__ and calls methods
    on those frames. Before the fix, this would cause a circular dependency:
    - Thread::GetStackFrameList() creates provider
    - Provider's __init__ accesses input_frames[0]
    - SBFrame::GetPC() tries to resolve ExecutionContextRef
    - ExecutionContextRef::GetFrameSP() calls Thread::GetStackFrameList()
    - Re-enters initialization -> circular dependency!

    With the fix, ExecutionContextRef remembers the frame list, so it doesn't
    re-enter Thread::GetStackFrameList().
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

        # This would cause circular dependency before the fix!
        # Accessing frames and calling methods on them during __init__
        self.original_frame_count = len(input_frames)
        self.original_pcs = []

        # Call GetPC() on each input frame - this triggers ExecutionContextRef resolution.
        for i in range(min(3, len(input_frames))):
            frame = input_frames[i]
            if frame.IsValid():
                pc = frame.GetPC()
                self.original_pcs.append(pc)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that tests circular dependency fix by accessing frames in __init__"

    def get_frame_at_index(self, index):
        """Prepend a synthetic frame, then pass through original frames."""
        if index == 0:
            # Synthetic frame at index 0.
            return {"pc": 0xDEADBEEF}
        elif index - 1 < self.original_frame_count:
            # Pass through original frames at indices 1, 2, 3, ...
            return index - 1
        return None


class PythonSourceFrame(ScriptedFrame):
    """Scripted frame that points to Python source code."""

    def __init__(self, thread, idx, function_name, python_file, line_number):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.function_name = function_name
        self.python_file = python_file
        self.line_number = line_number

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """PC-less frame - return invalid address."""
        return lldb.LLDB_INVALID_ADDRESS

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def get_symbol_context(self):
        """Return a symbol context with LineEntry pointing to Python source."""
        # Create a LineEntry pointing to the Python source file
        line_entry = lldb.SBLineEntry()
        line_entry.SetFileSpec(lldb.SBFileSpec(self.python_file, True))
        line_entry.SetLine(self.line_number)
        line_entry.SetColumn(0)

        # Create a symbol context with the line entry
        sym_ctx = lldb.SBSymbolContext()
        sym_ctx.SetLineEntry(line_entry)

        return sym_ctx

    def is_artificial(self):
        """Not artificial."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context for PC-less frames."""
        return None


class PythonSourceFrameProvider(ScriptedFrameProvider):
    """
    Provider that demonstrates Python source display in scripted frames.

    This provider prepends frames pointing to Python source code, showing
    that PC-less frames can display Python source files with proper line
    numbers and module/compile unit information.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

        # Find the python_helper.py file
        current_dir = os.path.dirname(os.path.abspath(__file__))
        self.python_file = os.path.join(current_dir, "python_helper.py")

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that prepends frames pointing to Python source"

    def get_frame_at_index(self, index):
        """Return Python source frames followed by original frames."""
        if index == 0:
            # Frame pointing to compute_fibonacci function (line 7)
            return PythonSourceFrame(
                self.thread, 0, "compute_fibonacci", self.python_file, 7
            )
        elif index == 1:
            # Frame pointing to process_data function (line 16)
            return PythonSourceFrame(
                self.thread, 1, "process_data", self.python_file, 16
            )
        elif index == 2:
            # Frame pointing to main function (line 27)
            return PythonSourceFrame(self.thread, 2, "main", self.python_file, 27)
        elif index - 3 < len(self.input_frames):
            # Pass through original frames
            return index - 3
        return None


class ValidPCNoModuleFrame(ScriptedFrame):
    """Scripted frame with a valid PC but no associated module."""

    def __init__(self, thread, idx, pc, function_name):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.pc = pc
        self.function_name = function_name

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """Return the program counter."""
        return self.pc

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def is_artificial(self):
        """Not artificial."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context."""
        return None


class ValidPCNoModuleFrameProvider(ScriptedFrameProvider):
    """
    Provider that demonstrates frames with valid PC but no module.

    This tests that backtrace output handles frames that have a valid
    program counter but cannot be resolved to any loaded module.
    """

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Provider that prepends frames with valid PC but no module"

    def get_frame_at_index(self, index):
        """Return frames with valid PCs but no module information."""
        if index == 0:
            # Frame with valid PC (0x1234000) but no module
            return ValidPCNoModuleFrame(self.thread, 0, 0x1234000, "unknown_function_1")
        elif index == 1:
            # Another frame with valid PC (0x5678000) but no module
            return ValidPCNoModuleFrame(self.thread, 1, 0x5678000, "unknown_function_2")
        elif index - 2 < len(self.input_frames):
            # Pass through original frames
            return index - 2
        return None


class AddFooFrameProvider(ScriptedFrameProvider):
    """Add a single 'foo' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'foo' frame at beginning"

    @staticmethod
    def get_priority():
        """Return priority 10 (runs first in chain)."""
        return 10

    def get_frame_at_index(self, index):
        if index == 0:
            # Return synthetic "foo" frame
            return CustomScriptedFrame(self.thread, 0, 0xF00, "foo")
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None


class AddBarFrameProvider(ScriptedFrameProvider):
    """Add a single 'bar' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'bar' frame at beginning"

    @staticmethod
    def get_priority():
        """Return priority 20 (runs second in chain)."""
        return 20

    def get_frame_at_index(self, index):
        if index == 0:
            # Return synthetic "bar" frame
            return CustomScriptedFrame(self.thread, 0, 0xBAA, "bar")
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None


class AddBazFrameProvider(ScriptedFrameProvider):
    """Add a single 'baz' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'baz' frame at beginning"

    @staticmethod
    def get_priority():
        """Return priority 30 (runs last in chain)."""
        return 30

    def get_frame_at_index(self, index):
        if index == 0:
            # Return synthetic "baz" frame
            return CustomScriptedFrame(self.thread, 0, 0xBAC, "baz")
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None


class ValueProvidingFrame(ScriptedFrame):
    """Scripted frame with a valid PC but no associated module."""

    def __init__(self, thread, idx, pc, function_name, variable):
        args = lldb.SBStructuredData()
        super().__init__(thread, args)

        self.idx = idx
        self.pc = pc
        self.function_name = function_name
        self.variable = variable

    def get_id(self):
        """Return the frame index."""
        return self.idx

    def get_pc(self):
        """Return the program counter."""
        return self.pc

    def get_function_name(self):
        """Return the function name."""
        return self.function_name

    def is_artificial(self):
        """Not artificial."""
        return False

    def is_hidden(self):
        """Not hidden."""
        return False

    def get_register_context(self):
        """No register context."""
        return None

    def get_variables(self):
        """"""
        out = lldb.SBValueList()
        out.Append(self.variable)
        return out

    def get_value_for_variable_expression(self, expr, options, error: lldb.SBError):
        out = lldb.SBValue()
        if expr == "_handler_one":
            out = self.variable.CreateValueFromExpression("_handler_one", "(uint32_t)1")
        elif self.variable.name in expr:
            out = self.variable.CreateValueFromExpression("_expr", expr)

        if out.IsValid():
            return out

        error.SetErrorString(f"expression {expr} failed")
        return None


class ValueProvidingFrameProvider(ScriptedFrameProvider):
    """Add a single 'value-provider' frame at the beginning."""

    def __init__(self, input_frames, args):
        super().__init__(input_frames, args)

    @staticmethod
    def get_description():
        """Return a description of this provider."""
        return "Add 'value-provider' frame at beginning"

    def get_frame_at_index(self, index):
        if index == 0:
            f = self.input_frames.GetFrameAtIndex(index)
            # Find some variable we can give to the frame.
            variable = f.FindVariable("variable_in_main")
            # Return synthetic "value-provider" frame
            return ValueProvidingFrame(
                self.thread, 0, 0xF00, "value-provider", variable
            )
        elif index - 1 < len(self.input_frames):
            # Pass through input frames (shifted by 1)
            return index - 1
        return None



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import xegpu
from mlir.dialects.transform import structured, AnyValueType


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f()
        print(module)
    return f


@run
def getDescOpDefaultIndex():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [0])
        desc_handle = xegpu.get_desc_op(operand)
        transform.YieldOp()
    # CHECK-LABEL: TEST: getDescOpDefaultIndex
    # CHECK: transform.xegpu.get_desc_op %


@run
def setDescLayoutMinimal():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.create_nd_tdesc"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_desc_layout(sequence.bodyTarget, sg_layout=[6, 4], sg_data=[32, 16])
        transform.YieldOp()
    # CHECK-LABEL: TEST: setDescLayoutMinimal
    # CHECK: %0 = transform.xegpu.set_desc_layout %
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]


@run
def setDescLayoutInstData():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.create_nd_tdesc"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_desc_layout(
            sequence.bodyTarget, sg_layout=[6, 4], sg_data=[32, 16], inst_data=[8, 16]
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setDescLayoutInstData
    # CHECK: %0 = transform.xegpu.set_desc_layout %
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: inst_data = [8, 16]


@run
def setDescLayoutSlice():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.create_nd_tdesc"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_desc_layout(
            sequence.bodyTarget, sg_layout=[6, 4], sg_data=[32, 16], slice_dims=[0]
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setDescLayoutSlice
    # CHECK: %0 = transform.xegpu.set_desc_layout %
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: slice_dims = [0]


@run
def setOpLayoutAttrOperandMinimal():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_op_layout_attr(
            sequence.bodyTarget,
            sg_layout=[6, 4],
            sg_data=[32, 16],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setOpLayoutAttr
    # CHECK: transform.xegpu.set_op_layout_attr %
    # NO-CHECK: index = 0
    # NO-CHECK: result
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # NO-CHECK: inst_data


@run
def setOpLayoutAttrResult():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_op_layout_attr(
            sequence.bodyTarget,
            index=0,
            sg_layout=[6, 4],
            sg_data=[32, 16],
            inst_data=[8, 16],
            result=True,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setOpLayoutAttrResult
    # CHECK: transform.xegpu.set_op_layout_attr %
    # NO-CHECK: index = 0
    # CHECK: result
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: inst_data = [8, 16]


@run
def setOpLayoutAttrResultSlice():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_op_layout_attr(
            sequence.bodyTarget,
            index=0,
            sg_layout=[6, 4],
            sg_data=[32, 16],
            inst_data=[8, 16],
            slice_dims=[0],
            result=True,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: setOpLayoutAttrResultSlice
    # CHECK: transform.xegpu.set_op_layout_attr %
    # NO-CHECK: index = 0
    # CHECK: result
    # CHECK: sg_layout = [6, 4]
    # CHECK: sg_data = [32, 16]
    # CHECK: inst_data = [8, 16]
    # CHECK: slice_dims = [0]


@run
def setGPULaunchThreadsOp():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("gpu.launch"),
    )
    with InsertionPoint(sequence.body):
        xegpu.set_gpu_launch_threads(sequence.bodyTarget, threads=[8, 4, 1])
        transform.YieldOp()
    # CHECK-LABEL: TEST: setGPULaunchThreadsOp
    # CHECK: transform.xegpu.set_gpu_launch_threads
    # CHECK: threads = [8, 4, 1]


@run
def insertPrefetch0():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [0])
        xegpu.insert_prefetch(
            operand,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: insertPrefetch0
    # CHECK: %[[OPR:.*]] = get_operand
    # CHECK: transform.xegpu.insert_prefetch %[[OPR]]


@run
def insertPrefetchNbPrefetch():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [0])
        xegpu.insert_prefetch(
            operand,
            nb_prefetch=2,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: insertPrefetchNbPrefetch
    # CHECK: %[[OPR:.*]] = get_operand
    # CHECK: transform.xegpu.insert_prefetch %[[OPR]]
    # CHECK-SAME: nb_prefetch = 2


@run
def insertPrefetchNbPrefetchParam():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [0])
        int32_t = IntegerType.get_signless(32)
        param_int32_t = transform.ParamType.get(int32_t)
        nb_param = transform.ParamConstantOp(
            param_int32_t,
            IntegerAttr.get(int32_t, 2),
        )
        xegpu.insert_prefetch(
            operand,
            nb_prefetch=nb_param,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: insertPrefetchNbPrefetchParam
    # CHECK: %[[OPR:.*]] = get_operand
    # CHECK: %[[PARAM_OP:.*]] = transform.param.constant 2
    # CHECK: transform.xegpu.insert_prefetch %[[OPR]]
    # CHECK-SAME: nb_prefetch = %[[PARAM_OP]]


@run
def ConvertLayoutMinimal():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [0])
        xegpu.convert_layout(
            operand,
            input_sg_layout=[6, 4],
            input_sg_data=[32, 16],
            target_sg_layout=[6, 4],
            target_sg_data=[8, 16],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: ConvertLayoutMinimal
    # CHECK: transform.xegpu.convert_layout %
    # CHECK: input_sg_layout = [6, 4]
    # CHECK: input_sg_data = [32, 16]
    # CHECK: target_sg_layout = [6, 4]
    # CHECK: target_sg_data = [8, 16]


@run
def ConvertLayout():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("xegpu.dpas"),
    )
    with InsertionPoint(sequence.body):
        operand = transform.GetOperandOp(AnyValueType.get(), sequence.bodyTarget, [1])
        xegpu.convert_layout(
            operand,
            input_sg_layout=[6, 4],
            input_sg_data=[32, 32],
            input_inst_data=[32, 16],
            target_sg_layout=[6, 4],
            target_sg_data=[32, 32],
            target_inst_data=[8, 16],
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: ConvertLayout
    # CHECK: transform.xegpu.convert_layout %
    # CHECK: input_sg_layout = [6, 4]
    # CHECK: input_sg_data = [32, 32]
    # CHECK: input_inst_data = [32, 16]
    # CHECK: target_sg_layout = [6, 4]
    # CHECK: target_sg_data = [32, 32]
    # CHECK: target_inst_data = [8, 16]



"""
Test lldb-dap runInTerminal reverse request
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import line_number
import lldbdap_testcase
import os
import subprocess
import json


@skipIfBuildType(["debug"])
class TestDAP_runInTerminal(lldbdap_testcase.DAPTestCaseBase):
    def read_pid_message(self, fifo_file):
        with open(fifo_file, "r") as file:
            self.assertIn("pid", file.readline())

    @staticmethod
    def send_did_attach_message(fifo_file):
        with open(fifo_file, "w") as file:
            file.write(json.dumps({"kind": "didAttach"}) + "\n")

    @staticmethod
    def read_error_message(fifo_file):
        with open(fifo_file, "r") as file:
            return file.readline()

    @skipIfAsan
    @skipIfWindows
    def test_runInTerminal(self):
        """
        Tests the "runInTerminal" reverse request. It makes sure that the IDE can
        launch the inferior with the correct environment variables and arguments.
        """
        program = self.getBuildArtifact("a.out")
        source = "main.c"
        self.build_and_launch(
            program, console="integratedTerminal", args=["foobar"], env=["FOO=bar"]
        )

        self.assertEqual(
            len(self.dap_server.reverse_requests),
            1,
            "make sure we got a reverse request",
        )

        request = self.dap_server.reverse_requests[0]
        self.assertIn(self.lldbDAPExec, request["arguments"]["args"])
        self.assertIn(program, request["arguments"]["args"])
        self.assertIn("foobar", request["arguments"]["args"])
        self.assertIn("FOO", request["arguments"]["env"])

        breakpoint_line = line_number(source, "// breakpoint")

        self.set_source_breakpoints(source, [breakpoint_line])
        self.continue_to_next_stop()

        # We verify we actually stopped inside the loop
        counter = int(self.dap_server.get_local_variable_value("counter"))
        self.assertEqual(counter, 1)

        # We verify we were able to set the launch arguments
        argc = int(self.dap_server.get_local_variable_value("argc"))
        self.assertEqual(argc, 2)

        argv1 = self.dap_server.request_evaluate("argv[1]")["body"]["result"]
        self.assertIn("foobar", argv1)

        # We verify we were able to set the environment
        env = self.dap_server.request_evaluate("foo")["body"]["result"]
        self.assertIn("bar", env)

        self.continue_to_exit()

    @skipIfAsan
    @skipIfWindows
    def test_runInTerminalWithObjectEnv(self):
        """
        Tests the "runInTerminal" reverse request. It makes sure that the IDE can
        launch the inferior with the correct environment variables using an object.
        """
        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program, console="integratedTerminal", env={"FOO": "BAR"})

        self.assertEqual(
            len(self.dap_server.reverse_requests),
            1,
            "make sure we got a reverse request",
        )

        request = self.dap_server.reverse_requests[0]
        request_envs = request["arguments"]["env"]

        self.assertIn("FOO", request_envs)
        self.assertEqual("BAR", request_envs["FOO"])

        self.continue_to_exit()

    @skipIfWindows
    def test_runInTerminalInvalidTarget(self):
        self.build_and_create_debug_adapter()
        response = self.launch(
            "INVALIDPROGRAM",
            console="integratedTerminal",
            args=["foobar"],
            env=["FOO=bar"],
            waitForResponse=True,
        )
        self.assertFalse(response["success"])
        self.assertIn(
            "'INVALIDPROGRAM' does not exist",
            response["body"]["error"]["format"],
        )

    @skipIfWindows
    def test_missingArgInRunInTerminalLauncher(self):
        proc = subprocess.run(
            [self.lldbDAPExec, "--launch-target", "INVALIDPROGRAM"],
            capture_output=True,
            universal_newlines=True,
        )
        self.assertNotEqual(proc.returncode, 0)
        self.assertIn(
            '"--launch-target" requires "--comm-file" to be specified', proc.stderr
        )

    @skipIfWindows
    def test_FakeAttachedRunInTerminalLauncherWithInvalidProgram(self):
        comm_file = os.path.join(self.getBuildDir(), "comm-file")
        os.mkfifo(comm_file)

        proc = subprocess.Popen(
            [
                self.lldbDAPExec,
                "--comm-file",
                comm_file,
                "--launch-target",
                "INVALIDPROGRAM",
            ],
            universal_newlines=True,
            stderr=subprocess.PIPE,
        )

        self.read_pid_message(comm_file)
        self.send_did_attach_message(comm_file)
        self.assertIn("No such file or directory", self.read_error_message(comm_file))

        _, stderr = proc.communicate()
        self.assertIn("No such file or directory", stderr)

    @skipIfWindows
    def test_FakeAttachedRunInTerminalLauncherWithValidProgram(self):
        comm_file = os.path.join(self.getBuildDir(), "comm-file")
        os.mkfifo(comm_file)

        proc = subprocess.Popen(
            [
                self.lldbDAPExec,
                "--comm-file",
                comm_file,
                "--launch-target",
                "echo",
                "foo",
            ],
            universal_newlines=True,
            stdout=subprocess.PIPE,
        )

        self.read_pid_message(comm_file)
        self.send_did_attach_message(comm_file)

        stdout, _ = proc.communicate()
        self.assertIn("foo", stdout)

    @skipIfWindows
    def test_FakeAttachedRunInTerminalLauncherAndCheckEnvironment(self):
        comm_file = os.path.join(self.getBuildDir(), "comm-file")
        os.mkfifo(comm_file)

        proc = subprocess.Popen(
            [self.lldbDAPExec, "--comm-file", comm_file, "--launch-target", "env"],
            universal_newlines=True,
            stdout=subprocess.PIPE,
            env={**os.environ, "FOO": "BAR"},
        )

        self.read_pid_message(comm_file)
        self.send_did_attach_message(comm_file)

        stdout, _ = proc.communicate()
        self.assertIn("FOO=BAR", stdout)

    @skipIfWindows
    def test_NonAttachedRunInTerminalLauncher(self):
        comm_file = os.path.join(self.getBuildDir(), "comm-file")
        os.mkfifo(comm_file)

        proc = subprocess.Popen(
            [
                self.lldbDAPExec,
                "--comm-file",
                comm_file,
                "--launch-target",
                "echo",
                "foo",
            ],
            universal_newlines=True,
            stderr=subprocess.PIPE,
            env={**os.environ, "LLDB_DAP_RIT_TIMEOUT_IN_MS": "1000"},
        )

        self.read_pid_message(comm_file)

        _, stderr = proc.communicate()
        self.assertIn("Timed out trying to get messages from the debug adapter", stderr)



"""
LLDB AppKit formatters

Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
See https://llvm.org/LICENSE.txt for license information.
SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""
# example summary provider for NSMachPort
# the real summary is now C++ code built into LLDB
import lldb
import ctypes
import lldb.runtime.objc.objc_runtime
import lldb.formatters.metrics
import lldb.formatters.Logger

statistics = lldb.formatters.metrics.Metrics()
statistics.add_metric("invalid_isa")
statistics.add_metric("invalid_pointer")
statistics.add_metric("unknown_class")
statistics.add_metric("code_notrun")

# despite the similary to synthetic children providers, these classes are not
# trying to provide anything but the port number of an NSMachPort, so they need not
# obey the interface specification for synthetic children providers


class NSMachPortKnown_SummaryProvider:
    def adjust_for_architecture(self):
        pass

    def __init__(self, valobj, params):
        logger = lldb.formatters.Logger.Logger()
        self.valobj = valobj
        self.sys_params = params
        if not (self.sys_params.types_cache.NSUInteger):
            if self.sys_params.is_64_bit:
                self.sys_params.types_cache.NSUInteger = (
                    self.valobj.GetType().GetBasicType(lldb.eBasicTypeUnsignedLong)
                )
            else:
                self.sys_params.types_cache.NSUInteger = (
                    self.valobj.GetType().GetBasicType(lldb.eBasicTypeUnsignedInt)
                )
        self.update()

    def update(self):
        logger = lldb.formatters.Logger.Logger()
        self.adjust_for_architecture()

    # one pointer is the ISA
    # then we have one other internal pointer, plus
    # 4 bytes worth of flags. hence, these values
    def offset(self):
        logger = lldb.formatters.Logger.Logger()
        if self.sys_params.is_64_bit:
            return 20
        else:
            return 12

    def port(self):
        logger = lldb.formatters.Logger.Logger()
        vport = self.valobj.CreateChildAtOffset(
            "port", self.offset(), self.sys_params.types_cache.NSUInteger
        )
        return vport.GetValueAsUnsigned(0)


class NSMachPortUnknown_SummaryProvider:
    def adjust_for_architecture(self):
        pass

    def __init__(self, valobj, params):
        logger = lldb.formatters.Logger.Logger()
        self.valobj = valobj
        self.sys_params = params
        self.update()

    def update(self):
        logger = lldb.formatters.Logger.Logger()
        self.adjust_for_architecture()

    def port(self):
        logger = lldb.formatters.Logger.Logger()
        stream = lldb.SBStream()
        self.valobj.GetExpressionPath(stream)
        num_children_vo = self.valobj.CreateValueFromExpression(
            "port", "(int)[" + stream.GetData() + " machPort]"
        )
        if num_children_vo.IsValid():
            return num_children_vo.GetValueAsUnsigned(0)
        return "<variable is not NSMachPort>"


def GetSummary_Impl(valobj):
    logger = lldb.formatters.Logger.Logger()
    global statistics
    (
        class_data,
        wrapper,
    ) = lldb.runtime.objc.objc_runtime.Utilities.prepare_class_detection(
        valobj, statistics
    )
    if wrapper:
        return wrapper

    name_string = class_data.class_name()
    logger >> "class name is: " + str(name_string)

    if name_string == "NSMachPort":
        wrapper = NSMachPortKnown_SummaryProvider(valobj, class_data.sys_params)
        statistics.metric_hit("code_notrun", valobj)
    else:
        wrapper = NSMachPortUnknown_SummaryProvider(valobj, class_data.sys_params)
        statistics.metric_hit(
            "unknown_class", valobj.GetName() + " seen as " + name_string
        )
    return wrapper


def NSMachPort_SummaryProvider(valobj, dict):
    logger = lldb.formatters.Logger.Logger()
    provider = GetSummary_Impl(valobj)
    if provider is not None:
        if isinstance(
            provider, lldb.runtime.objc.objc_runtime.SpecialSituation_Description
        ):
            return provider.message()
        try:
            summary = provider.port()
        except:
            summary = None
        logger >> "got summary " + str(summary)
        if summary is None:
            summary = "<variable is not NSMachPort>"
        if isinstance(summary, str):
            return summay
        return "mach port: " + str(summary)
    return "Summary Unavailable"


def __lldb_init_module(debugger, dict):
    debugger.HandleCommand(
        "type summary add -F NSMachPort.NSMachPort_SummaryProvider NSMachPort"
    )



"""
Specification, compiler, disassembler, and interpreter
for LLDB dataformatter bytecode.

See https://lldb.llvm.org/resources/formatterbytecode.html for more details.
"""

from __future__ import annotations

# Types
type_String = 1
type_Int = 2
type_UInt = 3
type_Object = 4
type_Type = 5

# Opcodes
opcode = dict()


def define_opcode(n, mnemonic, name):
    globals()["op_" + name] = n
    if mnemonic:
        opcode[mnemonic] = n
    opcode[n] = mnemonic


define_opcode(1, "dup", "dup")
define_opcode(2, "drop", "drop")
define_opcode(3, "pick", "pick")
define_opcode(4, "over", "over")
define_opcode(5, "swap", "swap")
define_opcode(6, "rot", "rot")

define_opcode(0x10, "{", "begin")
define_opcode(0x11, "if", "if")
define_opcode(0x12, "ifelse", "ifelse")
define_opcode(0x13, "return", "return")

define_opcode(0x20, None, "lit_uint")
define_opcode(0x21, None, "lit_int")
define_opcode(0x22, None, "lit_string")
define_opcode(0x23, None, "lit_selector")

define_opcode(0x2A, "as_int", "as_int")
define_opcode(0x2B, "as_uint", "as_uint")
define_opcode(0x2C, "is_null", "is_null")

define_opcode(0x30, "+", "plus")
define_opcode(0x31, "-", "minus")
define_opcode(0x32, "*", "mul")
define_opcode(0x33, "/", "div")
define_opcode(0x34, "%", "mod")
define_opcode(0x35, "<<", "shl")
define_opcode(0x36, ">>", "shr")

define_opcode(0x40, "&", "and")
define_opcode(0x41, "|", "or")
define_opcode(0x42, "^", "xor")
define_opcode(0x43, "~", "not")

define_opcode(0x50, "=", "eq")
define_opcode(0x51, "!=", "neq")
define_opcode(0x52, "<", "lt")
define_opcode(0x53, ">", "gt")
define_opcode(0x54, "=<", "le")
define_opcode(0x55, ">=", "ge")

define_opcode(0x60, "call", "call")

# Function signatures
sig_summary = 0
sig_init = 1
sig_get_num_children = 2
sig_get_child_index = 3
sig_get_child_at_index = 4

# Selectors
selector = dict()


def define_selector(n, name):
    globals()["sel_" + name] = n
    selector["@" + name] = n
    selector[n] = "@" + name


define_selector(0, "summary")
define_selector(1, "type_summary")

define_selector(0x10, "get_num_children")
define_selector(0x11, "get_child_at_index")
define_selector(0x12, "get_child_with_name")
define_selector(0x13, "get_child_index")
define_selector(0x15, "get_type")
define_selector(0x16, "get_template_argument_type")
define_selector(0x17, "cast")
define_selector(0x20, "get_value")
define_selector(0x21, "get_value_as_unsigned")
define_selector(0x22, "get_value_as_signed")
define_selector(0x23, "get_value_as_address")

define_selector(0x40, "read_memory_byte")
define_selector(0x41, "read_memory_uint32")
define_selector(0x42, "read_memory_int32")
define_selector(0x43, "read_memory_unsigned")
define_selector(0x44, "read_memory_signed")
define_selector(0x45, "read_memory_address")
define_selector(0x46, "read_memory")

define_selector(0x50, "fmt")
define_selector(0x51, "sprintf")
define_selector(0x52, "strlen")


################################################################################
# Compiler.
################################################################################


def compile(assembler: str) -> bytearray:
    """Compile assembler into bytecode"""
    # This is a stack of all in-flight/unterminated blocks.
    bytecode = [bytearray()]

    def emit(byte):
        bytecode[-1].append(byte)

    tokens = list(assembler.split(" "))
    tokens.reverse()
    while tokens:
        tok = tokens.pop()
        if tok == "":
            pass
        elif tok == "{":
            bytecode.append(bytearray())
        elif tok == "}":
            block = bytecode.pop()
            emit(op_begin)
            emit(len(block))  # FIXME: uleb
            bytecode[-1].extend(block)
        elif tok[0].isdigit():
            if tok[-1] == "u":
                emit(op_lit_uint)
                emit(int(tok[:-1]))  # FIXME
            else:
                emit(op_lit_int)
                emit(int(tok))  # FIXME
        elif tok[0] == "@":
            emit(op_lit_selector)
            emit(selector[tok])
        elif tok[0] == '"':
            s = bytearray()
            done = False
            chrs = tok[1:]
            while not done:
                quoted = False
                for c in chrs:
                    if quoted:
                        s.append(ord(c))  # FIXME
                        quoted = False
                    elif c == "\\":
                        quoted = True
                    elif c == '"':
                        done = True
                        break
                        # FIXME assert this is last in token
                    else:
                        s.append(ord(c))
                if not done:
                    s.append(ord(" "))
                    chrs = tokens.pop()

            emit(op_lit_string)
            emit(len(s))
            bytecode[-1].extend(s)
        else:
            emit(opcode[tok])
    assert len(bytecode) == 1  # unterminated {
    return bytecode[0]


################################################################################
# Disassembler.
################################################################################


def disassemble(bytecode: bytearray) -> (str, int):
    """Disassemble bytecode into (assembler, token starts)"""
    asm = ""
    all_bytes = list(bytecode)
    all_bytes.reverse()
    blocks = []
    tokens = [0]

    def next_byte():
        """Fetch the next byte in the bytecode and keep track of all
        in-flight blocks"""
        for i in range(len(blocks)):
            blocks[i] -= 1
        tokens.append(len(asm))
        return all_bytes.pop()

    while all_bytes:
        b = next_byte()
        if b == op_begin:
            asm += "{"
            length = next_byte()
            blocks.append(length)
        elif b == op_lit_uint:
            b = next_byte()
            asm += str(b)  # FIXME uleb
            asm += "u"
        elif b == op_lit_int:
            b = next_byte()
            asm += str(b)
        elif b == op_lit_selector:
            b = next_byte()
            asm += selector[b]
        elif b == op_lit_string:
            length = next_byte()
            s = "'"
            while length:
                s += chr(next_byte())
                length -= 1
            asm += '"' + repr(s)[2:]
        else:
            asm += opcode[b]

        while blocks and blocks[-1] == 0:
            asm += " }"
            blocks.pop()

        if all_bytes:
            asm += " "

    if blocks:
        asm += "ERROR"
    return asm, tokens


################################################################################
# Interpreter.
################################################################################


def count_fmt_params(fmt: str) -> int:
    """Count the number of parameters in a format string"""
    from string import Formatter

    f = Formatter()
    n = 0
    for _, name, _, _ in f.parse(fmt):
        if name > n:
            n = name
    return n


def interpret(bytecode: bytearray, control: list, data: list, tracing: bool = False):
    """Interpret bytecode"""
    frame = []
    frame.append((0, len(bytecode)))

    def trace():
        """print a trace of the execution for debugging purposes"""

        def fmt(d):
            if isinstance(d, int):
                return str(d)
            if isinstance(d, str):
                return d
            return repr(type(d))

        pc, end = frame[-1]
        asm, tokens = disassemble(bytecode)
        print(
            "=== frame = {1}, data = {2}, opcode = {0}".format(
                opcode[b], frame, [fmt(d) for d in data]
            )
        )
        print(asm)
        print(" " * (tokens[pc]) + "^")

    def next_byte():
        """Fetch the next byte and update the PC"""
        pc, end = frame[-1]
        assert pc < len(bytecode)
        b = bytecode[pc]
        frame[-1] = pc + 1, end
        # At the end of a block?
        while pc >= end:
            frame.pop()
            if not frame:
                return None
            pc, end = frame[-1]
            if pc >= end:
                return None
            b = bytecode[pc]
            frame[-1] = pc + 1, end
        return b

    while frame[-1][0] < len(bytecode):
        b = next_byte()
        if b == None:
            break
        if tracing:
            trace()
        # Data stack manipulation.
        if b == op_dup:
            data.append(data[-1])
        elif b == op_drop:
            data.pop()
        elif b == op_pick:
            data.append(data[data.pop()])
        elif b == op_over:
            data.append(data[-2])
        elif b == op_swap:
            x = data.pop()
            y = data.pop()
            data.append(x)
            data.append(y)
        elif b == op_rot:
            z = data.pop()
            y = data.pop()
            x = data.pop()
            data.append(z)
            data.append(x)
            data.append(y)

        # Control stack manipulation.
        elif b == op_begin:
            length = next_byte()
            pc, end = frame[-1]
            control.append((pc, pc + length))
            frame[-1] = pc + length, end
        elif b == op_if:
            if data.pop():
                frame.append(control.pop())
        elif b == op_ifelse:
            if data.pop():
                control.pop()
                frame.append(control.pop())
            else:
                frame.append(control.pop())
                control.pop()
        elif b == op_return:
            control.clear()
            return data[-1]

        # Literals.
        elif b == op_lit_uint:
            b = next_byte()  # FIXME uleb
            data.append(int(b))
        elif b == op_lit_int:
            b = next_byte()  # FIXME uleb
            data.append(int(b))
        elif b == op_lit_selector:
            b = next_byte()
            data.append(b)
        elif b == op_lit_string:
            length = next_byte()
            s = ""
            while length:
                s += chr(next_byte())
                length -= 1
            data.append(s)

        elif b == op_as_uint:
            pass
        elif b == op_as_int:
            pass
        elif b == op_is_null:
            data.append(1 if data.pop() == None else 0)

        # Arithmetic, logic, etc.
        elif b == op_plus:
            data.append(data.pop() + data.pop())
        elif b == op_minus:
            data.append(-data.pop() + data.pop())
        elif b == op_mul:
            data.append(data.pop() * data.pop())
        elif b == op_div:
            y = data.pop()
            data.append(data.pop() / y)
        elif b == op_mod:
            y = data.pop()
            data.append(data.pop() % y)
        elif b == op_shl:
            y = data.pop()
            data.append(data.pop() << y)
        elif b == op_shr:
            y = data.pop()
            data.append(data.pop() >> y)
        elif b == op_and:
            data.append(data.pop() & data.pop())
        elif b == op_or:
            data.append(data.pop() | data.pop())
        elif b == op_xor:
            data.append(data.pop() ^ data.pop())
        elif b == op_not:
            data.append(not data.pop())
        elif b == op_eq:
            data.append(data.pop() == data.pop())
        elif b == op_neq:
            data.append(data.pop() != data.pop())
        elif b == op_lt:
            data.append(data.pop() > data.pop())
        elif b == op_gt:
            data.append(data.pop() < data.pop())
        elif b == op_le:
            data.append(data.pop() >= data.pop())
        elif b == op_ge:
            data.append(data.pop() <= data.pop())

        # Function calls.
        elif b == op_call:
            sel = data.pop()
            if sel == sel_summary:
                data.append(data.pop().GetSummary())
            elif sel == sel_get_num_children:
                data.append(data.pop().GetNumChildren())
            elif sel == sel_get_child_at_index:
                index = data.pop()
                valobj = data.pop()
                data.append(valobj.GetChildAtIndex(index))
            elif sel == sel_get_child_with_name:
                name = data.pop()
                valobj = data.pop()
                data.append(valobj.GetChildMemberWithName(name))
            elif sel == sel_get_child_index:
                name = data.pop()
                valobj = data.pop()
                data.append(valobj.GetIndexOfChildWithName(name))
            elif sel == sel_get_type:
                data.append(data.pop().GetType())
            elif sel == sel_get_template_argument_type:
                n = data.pop()
                valobj = data.pop()
                data.append(valobj.GetTemplateArgumentType(n))
            elif sel == sel_get_value:
                data.append(data.pop().GetValue())
            elif sel == sel_get_value_as_unsigned:
                data.append(data.pop().GetValueAsUnsigned())
            elif sel == sel_get_value_as_signed:
                data.append(data.pop().GetValueAsSigned())
            elif sel == sel_get_value_as_address:
                data.append(data.pop().GetValueAsAddress())
            elif sel == sel_cast:
                sbtype = data.pop()
                valobj = data.pop()
                data.append(valobj.Cast(sbtype))
            elif sel == sel_strlen:
                s = data.pop()
                data.append(len(s) if s else 0)
            elif sel == sel_fmt:
                fmt = data.pop()
                n = count_fmt_params(fmt)
                args = []
                for i in range(n):
                    args.append(data.pop())
                data.append(fmt.format(*args))
            else:
                print("not implemented: " + selector[sel])
                assert False
                pass
    return data[-1]


if __name__ == "__main__":
    # Work around the fact that one of the local files is called
    # types.py, which breaks some versions of python.
    import os, sys

    path = os.path.abspath(os.path.dirname(__file__))
    sys.path.remove(path)
    import argparse

    parser = argparse.ArgumentParser(
        description="""
    Compiler, disassembler, and interpreter for LLDB dataformatter bytecode.
    See https://lldb.llvm.org/resources/formatterbytecode.html for more details.
    """
    )
    parser.add_argument(
        "-c", "--compile", type=str, help="compile assembler into bytecode"
    )
    parser.add_argument("-d", "--disassemble", type=str, help="disassemble bytecode")
    parser.add_argument("-t", "--test", action="store_true", help="run unit tests")
    args = parser.parse_args()
    if args.compile:
        print(compile(str(args.compile)).hex())

    if args.disassemble:
        print(disassemble(bytearray.fromhex(str(args.disassemble))))

    ############################################################################
    # Tests.
    ############################################################################
    if args.test:
        import unittest

        class TestCompiler(unittest.TestCase):
            def test(self):
                self.assertEqual(compile("1u dup").hex(), "200101")
                self.assertEqual(compile('"1u dup"').hex(), "2206317520647570")
                self.assertEqual(compile("16 < { dup } if").hex(), "21105210010111")
                self.assertEqual(compile('{ { " } " } }').hex(), "100710052203207d20")

                def roundtrip(asm):
                    self.assertEqual(disassemble(compile(asm))[0], asm)

                roundtrip("1u dup")
                roundtrip('1u dup "1u dup"')
                roundtrip("16 < { dup } if")
                roundtrip('{ { " } " } }')

                self.assertEqual(interpret(compile("1 1 +"), [], []), 2)
                self.assertEqual(interpret(compile("2 1 1 + *"), [], []), 4)
                self.assertEqual(
                    interpret(compile('2 1 > { "yes" } { "no" } ifelse'), [], []), "yes"
                )

                import sys

                sys.argv.pop()
                path = os.path.dirname(__file__)
                sys.path.remove
                unittest.main()
